searchData={"items":[{"type":"module","title":"Xema","doc":"A schema validator inspired by [JSON Schema](http://json-schema.org).\n\nAll available keywords to construct a schema are described on page\n[Usage](usage.html).\n\nThis module can be used to construct a schema module. Should a module\ncontain multiple schemas the option `multi: true` is required.\n\n`use Xema` imports `Xema.Builder` and extends the module with the functions\n+ `__MODULE__.valid?/2`\n+ `__MODULE__.validate/2`\n+ `__MODULE__.validate!/2`\n+ `__MODULE__.cast/2`\n+ `__MODULE__.cast!/2`\n+ `__MODULE__.xema/1`\n\nThe macro `xema/2` supports the construction of a schema. After that\nthe schema is available via the functions above.\n\nIn a multi schema module a schema can be tagged with the option\n`default: :schema_name` and then called by\n+ `__MODULE__.valid?/1`\n+ `__MODULE__.validate/1`\n+ `__MODULE__.validate!/1`\n+ `__MODULE__.cast/1`\n+ `__MODULE__.cast!/1`\n+ `__MODULE__.xema/0`\n\nThe functions with arity 1 are also available for single schema modules.\n\nThe macro `xema_struct/1` creates a schema with the coresponding struct.","ref":"Xema.html"},{"type":"module","title":"Examples - Xema","doc":"Single schema module:\n\n    iex> defmodule SingleSchema do\n    ...>   use Xema\n    ...>\n    ...>   # The name :num is optional.\n    ...>   xema :num, do: number(minimum: 1)\n    ...> end\n    iex>\n    iex> SingleSchema.valid?(:num, 6)\n    true\n    iex> SingleSchema.valid?(5)\n    true\n    iex> SingleSchema.validate(0)\n    {:error, %Xema.ValidationError{\n       reason: %{minimum: 1, value: 0}\n    }}\n    iex> SingleSchema.cast(\"5\")\n    {:ok, 5}\n    iex> SingleSchema.cast(\"-5\")\n    {:error, %Xema.ValidationError{\n       reason: %{minimum: 1, value: -5}\n    }}\n\nMulti schema module:\n\n    iex> defmodule Schema do\n    ...>   use Xema, multi: true, default: :user\n    ...>\n    ...>   @pos integer(minimum: 0)\n    ...>   @neg integer(maximum: 0)\n    ...>\n    ...>   xema :user do\n    ...>     map(\n    ...>       properties: %{\n    ...>         name: string(min_length: 1),\n    ...>         age: @pos\n    ...>       }\n    ...>     )\n    ...>   end\n    ...>\n    ...>   xema :nums do\n    ...>     map(\n    ...>       properties: %{\n    ...>         pos: list(items: @pos),\n    ...>         neg: list(items: @neg)\n    ...>       }\n    ...>     )\n    ...>   end\n    ...> end\n    iex>\n    iex> Schema.valid?(:user, %{name: \"John\", age: 21})\n    true\n    iex> Schema.valid?(%{name: \"John\", age: 21})\n    true\n    iex> Schema.valid?(%{name: \"\", age: 21})\n    false\n    iex> Schema.validate(%{name: \"John\", age: 21})\n    :ok\n    iex> Schema.validate(%{name: \"\", age: 21})\n    {:error, %Xema.ValidationError{\n      reason: %{\n        properties: %{name: %{min_length: 1, value: \"\"}}}\n      }\n    }\n    iex> Schema.valid?(:nums, %{pos: [1, 2, 3]})\n    true\n    iex> Schema.valid?(:nums, %{neg: [1, 2, 3]})\n    false\n\nStruct schema module:\n\n    iex> defmodule StructA do\n    ...>   use Xema\n    ...>\n    ...>   xema_struct do\n    ...>     field :foo, :integer, minimum: 0\n    ...>   end\n    ...> end\n    ...>\n    ...> defmodule StructB do\n    ...>   use Xema\n    ...>\n    ...>   xema_struct do\n    ...>     field :a, :string, min_length: 3\n    ...>     field :b, StructA\n    ...>     required [:a]\n    ...>   end\n    ...> end\n    ...>\n    ...> data = StructB.cast!(a: \"abc\", b: %{foo: 5})\n    ...> data.a\n    \"abc\"\n    iex> Map.from_struct(data.b)\n    %{foo: 5}\n\nFor more examples to construct schemas see \"[Examples](examples.html)\".","ref":"Xema.html#module-examples"},{"type":"function","title":"Xema.cast/3","doc":"Converts the given data using the specified schema. Returns `{:ok, result}` or\n`{:error, reason}`. The `result` is converted and validated with the schema.","ref":"Xema.html#cast/3"},{"type":"function","title":"Examples: - Xema.cast/3","doc":"iex> schema = Xema.new({:integer, minimum: 1})\n    iex> Xema.cast(schema, \"5\")\n    {:ok, 5}\n    iex> Xema.cast(schema, \"five\")\n    {:error, %Xema.CastError{\n      key: nil,\n      path: [],\n      to: :integer,\n      value: \"five\"\n    }}\n    iex> Xema.cast(schema, \"0\")\n    {:error, %Xema.ValidationError{\n      reason: %{minimum: 1, value: 0}\n    }}","ref":"Xema.html#cast/3-examples"},{"type":"function","title":"Multiple types - Xema.cast/3","doc":"If for a value multiple types are defined the function used the result of the\nfirst successful conversion.","ref":"Xema.html#cast/3-multiple-types"},{"type":"function","title":"Examples - Xema.cast/3","doc":"iex> schema = Xema.new([:integer, :string, nil])\n    iex> Xema.cast(schema, 5)\n    {:ok, 5}\n    iex> Xema.cast(schema, 5.5)\n    {:ok, \"5.5\"}\n    iex> Xema.cast(schema, \"5\")\n    {:ok, 5}\n    iex> Xema.cast(schema, \"five\")\n    {:ok, \"five\"}\n    iex> Xema.cast(schema, nil)\n    {:ok, nil}\n    iex> Xema.cast(schema, [5])\n    {:error,\n      %Xema.CastError{path: [], to: [:integer, :string, nil], value: [5]}\n    }","ref":"Xema.html#cast/3-examples"},{"type":"function","title":"Cast with `any_of`, `all_of`, and `one_of` - Xema.cast/3","doc":"Schemas in a combiner will be cast independently one by one in reverse order.","ref":"Xema.html#cast/3-cast-with-any_of-all_of-and-one_of"},{"type":"function","title":"Examples - Xema.cast/3","doc":"iex> schema = Xema.new(any_of: [\n    ...>   [properties: %{a: :integer}],\n    ...>   [properties: %{a: :string}]\n    ...> ])\n    iex> Xema.cast(schema, %{a: 5})\n    {:ok, %{a: 5}}\n    iex> Xema.cast(schema, %{a: 5.5})\n    {:ok, %{a: \"5.5\"}}\n    iex> Xema.cast(schema, %{a: \"5\"})\n    {:ok, %{a: 5}}\n    iex> Xema.cast(schema, %{a: \"five\"})\n    {:ok, %{a: \"five\"}}\n    iex> Xema.cast(schema, %{a: [5]})\n    {:error,\n      %Xema.CastError{\n        error: nil,\n        key: nil,\n        message: nil,\n        path: [],\n        to: [\n          %{path: [:a], to: :integer, value: [5]},\n          %{path: [:a], to: :string, value: [5]}\n        ],\n        value: %{a: [5]}\n    }}","ref":"Xema.html#cast/3-examples"},{"type":"function","title":"Options - Xema.cast/3","doc":"With the option `additional_properties: :delete` additional properties will be\ndeleted on cast. Additional properties will be deleted in schemas with\n`additional_properties: false`.","ref":"Xema.html#cast/3-options"},{"type":"function","title":"Examples - Xema.cast/3","doc":"iex> schema = Xema.new(\n    ...>   properties: %{\n    ...>     a: [\n    ...>       properties: %{\n    ...>         foo: :integer\n    ...>       },\n    ...>       additional_properties: false\n    ...>     ],\n    ...>     b: [\n    ...>       properties: %{\n    ...>         foo: :integer\n    ...>       }\n    ...>     ]\n    ...>   }\n    ...> )\n    iex>\n    iex> Xema.cast(schema, %{\n    ...>   a: %{foo: \"6\", bar: \"7\"},\n    ...>   b: %{foo: \"6\", bar: \"7\"},\n    ...> }, additional_properties: :delete)\n    {:ok, %{\n      a: %{foo: 6},\n      b: %{foo: 6, bar: \"7\"}\n    }}","ref":"Xema.html#cast/3-examples"},{"type":"function","title":"Xema.cast!/3","doc":"Converts the given data using the specified schema. Returns the converted data\nor an exception.","ref":"Xema.html#cast!/3"},{"type":"function","title":"Xema.from_json_schema/2","doc":"Creates a `Xema` from a JSON Schema.  The argument `json_schema` is expected\nas a decoded JSON Schema.\n\nAll keys that are not standard JSON Schema keywords have to be known atoms. If\nthe schema has additional keys that are unknown atoms the option\n`atom: :force` is needed. In this case the atoms will be created. This is not\nneeded for keys expected by JSON Schema (e.g. in properties)\n\nOptions:\n* `:draft` specifies the draft to check the given JSON Schema. Possible values\n  are `\"draft4\"`, `\"draft6\"`, and `\"draft7\"`, default is `\"draft7\"`. If\n  `:draft` not set and the schema contains `$schema` then the value for\n  `$schema` is used for this option.\n* `:atom` creates atoms for unknown atoms when set to `:force`. This is just\n  needed for additional JSON Schema keywords.","ref":"Xema.html#from_json_schema/2"},{"type":"function","title":"Examples - Xema.from_json_schema/2","doc":"iex> Xema.from_json_schema(%{\"type\" => \"integer\", \"minimum\" => 5})\n    %Xema{schema: %Xema.Schema{minimum: 5, type: :integer}}\n\n    iex> schema = %{\n    ...>   \"type\" => \"object\",\n    ...>   \"properties\" => %{\"foo\" => %{\"type\" => \"integer\"}}\n    ...> }\n    iex> Xema.from_json_schema(schema)\n    %Xema{schema:\n      %Xema.Schema{\n        properties: %{\"foo\" => %Xema.Schema{type: :integer}},\n        type: :map,\n        keys: :strings\n      }\n    }\n\n    iex> Xema.from_json_schema(%{\"type\" => \"integer\", \"foo\" => \"bar\"}, atom: :force)\n    %Xema{schema: %Xema.Schema{data: %{foo: \"bar\"}, type: :integer}}\n\n    iex> Xema.from_json_schema(%{\"exclusiveMaximum\" => 5}, draft: \"draft7\")\n    %Xema{schema: %Xema.Schema{exclusive_maximum: 5}}\n\n    iex> Xema.from_json_schema(%{\"exclusiveMaximum\" => 5}, draft: \"draft4\")\n    ** (Xema.SchemaError) Can't build schema:\n    Dependencies for \"exclusiveMaximum\" failed. Missing required key \"maximum\".","ref":"Xema.html#from_json_schema/2-examples"},{"type":"function","title":"Xema.new/2","doc":"This function creates the schema from the given `data`.\n\nPossible options:\n+ `:loader` - a loader for remote schemas. This option will overwrite the\n              loader from the config.\n              See [Configure a loader](loader.html) to how to define a loader.\n\n+ `inline` - inlined all references in the schema. Default `:true`.","ref":"Xema.html#new/2"},{"type":"function","title":"Examples - Xema.new/2","doc":"Simple schema:\n\n    iex> schema = Xema.new :string\n    iex> Xema.valid? schema, \"hello\"\n    true\n    iex> Xema.valid? schema, 42\n    false\n\nSchema:\n\n    iex> schema = Xema.new {:string, min_length: 3, max_length: 12}\n    iex> Xema.valid? schema, \"hello\"\n    true\n    iex> Xema.valid? schema, \"hi\"\n    false\n\nNested schemas:\n\n    iex> schema = Xema.new {:list, items: {:number, minimum: 2}}\n    iex> Xema.validate(schema, [2, 3, 4])\n    :ok\n    iex> Xema.valid?(schema, [2, 3, 4])\n    true\n    iex> Xema.validate(schema, [2, 3, 1])\n    {:error, %Xema.ValidationError{\n      reason: %{\n        items: %{2 => %{value: 1, minimum: 2}}}\n      }\n    }\n\nMore examples can be found on page\n[Usage](https://hexdocs.pm/xema/usage.html#content).","ref":"Xema.html#new/2-examples"},{"type":"type","title":"Xema.result/0","doc":"The return type of a validation run.","ref":"Xema.html#t:result/0"},{"type":"type","title":"Xema.schema/0","doc":"","ref":"Xema.html#t:schema/0"},{"type":"function","title":"Xema.source/1","doc":"Returns the source for a given `xema`. The output can differ from the input\nif the schema contains references. To get the original source the schema\nmust be created with `inline: false`.","ref":"Xema.html#source/1"},{"type":"function","title":"Examples - Xema.source/1","doc":"iex> {:integer, minimum: 1} |> Xema.new() |> Xema.source()\n    {:integer, minimum: 1}","ref":"Xema.html#source/1-examples"},{"type":"type","title":"Xema.t/0","doc":"This struct contains the schema and references of the schema.","ref":"Xema.html#t:t/0"},{"type":"function","title":"Xema.valid?/2","doc":"Returns `true` if the `value` is a valid value against the given `schema`;\notherwise returns `false`.","ref":"Xema.html#valid?/2"},{"type":"function","title":"Xema.validate/3","doc":"Returns `:ok` if the `value` is a valid value against the given `schema`;\notherwise returns an error tuple.\n\nWith the option `:fail`, you can define when the validation is aborted. This\nalso influences how many error reasons are returned.\n- `:immediately` aborts the validation when the first validation fails.\n- `:early` (default) aborts on failed validations, but runs validations\n  for all properties and items.\n- `:finally` aborts after all possible validations.","ref":"Xema.html#validate/3"},{"type":"function","title":"Xema.validate!/3","doc":"Returns `:ok` if the `value` is a valid value against the given `schema`;\notherwise raises a `Elixir.Xema.ValidationError`. See `validate3` for\navailable options.","ref":"Xema.html#validate!/3"},{"type":"behaviour","title":"Xema.Behaviour","doc":"A behaviour module for implementing a schema validator. This behaviour is\njust for `Xema` and `JsonXema`.","ref":"Xema.Behaviour.html"},{"type":"callback","title":"Xema.Behaviour.init/2","doc":"This callback initialize the schema. The function gets the data given to\n`Xema.new/1` and returns a `Xema.Schema`.","ref":"Xema.Behaviour.html#c:init/2"},{"type":"type","title":"Xema.Behaviour.t/0","doc":"The schema container.","ref":"Xema.Behaviour.html#t:t/0"},{"type":"module","title":"Xema.Builder","doc":"This module contains some convenience functions to generate schemas. Beside\nthe type-functions the module contains the combinator-functions `all_of/2`,\n`any_of/2` and `one_of/2`.","ref":"Xema.Builder.html"},{"type":"module","title":"Examples - Xema.Builder","doc":"iex> import Xema.Builder\n    ...> schema = Xema.new integer(minimum: 1)\n    ...> Xema.valid?(schema, 6)\n    true\n    ...> Xema.valid?(schema, 0)\n    false","ref":"Xema.Builder.html#module-examples"},{"type":"function","title":"Xema.Builder.all_of/2","doc":"Returns a tuple with the given `type` (default `:any`) and the given schemas\ntagged by the keyword `:all_of`. This function provides a shortcut for\nsomething like `integer(all_of: [...])` or `any(all_of: [...])`.","ref":"Xema.Builder.html#all_of/2"},{"type":"function","title":"Examples - Xema.Builder.all_of/2","doc":"iex> Xema.Builder.all_of([:integer, :string])\n    {:any, all_of: [:integer, :string] }\n\n    iex> Xema.Builder.all_of(:integer, [[minimum: 10], [maximum: 5]])\n    {:integer, all_of: [[minimum: 10], [maximum: 5]]}\n\n```elixir\ndefmodule MySchema do\n  use Xema\n\n  xema do\n    all_of [\n      list(items: integer(minimum: 1, maximum: 66)),\n      list(items: integer(minimum: 33, maximum: 100))\n    ]\n  end\nend\n\nMySchema.valid?([20, 30]) #=> false\nMySchema.valid?([40, 50]) #=> true\nMySchema.valid?([60, 70]) #=> false\nMySchema.valid?([10, 90]) #=> false\n```","ref":"Xema.Builder.html#all_of/2-examples"},{"type":"function","title":"Xema.Builder.any/0","doc":"Returns :any.","ref":"Xema.Builder.html#any/0"},{"type":"function","title":"Xema.Builder.any/1","doc":"Returns a tuple of `:any` and the given keyword list.","ref":"Xema.Builder.html#any/1"},{"type":"function","title":"Examples - Xema.Builder.any/1","doc":"iex> Xema.Builder.any(key: 42)\n    {:any, key: 42}","ref":"Xema.Builder.html#any/1-examples"},{"type":"function","title":"Xema.Builder.any_of/2","doc":"Returns a tuple with the given `type` (default `:any`) and the given schemas\ntagged by the keyword `:any_of`. This function provides a shortcut for\nsomething like `integer(any_of: [...])` or `any(any_of: [...])`.","ref":"Xema.Builder.html#any_of/2"},{"type":"function","title":"Examples - Xema.Builder.any_of/2","doc":"iex> Xema.Builder.any_of([:integer, :string])\n    {:any, any_of: [:integer, :string] }\n\n    iex> Xema.Builder.any_of(:integer, [[minimum: 10], [maximum: 5]])\n    {:integer, any_of: [[minimum: 10], [maximum: 5]]}\n\n```elixir\ndefmodule MySchema do\n  use Xema\n\n  xema do\n    any_of [\n      list(items: integer(minimum: 1, maximum: 66)),\n      list(items: integer(minimum: 33, maximum: 100))\n    ]\n  end\nend\n\nMySchema.valid?([20, 30]) #=> true\nMySchema.valid?([40, 50]) #=> true\nMySchema.valid?([60, 70]) #=> true\nMySchema.valid?([10, 90]) #=> false\n```","ref":"Xema.Builder.html#any_of/2-examples"},{"type":"function","title":"Xema.Builder.atom/0","doc":"Returns :atom.","ref":"Xema.Builder.html#atom/0"},{"type":"function","title":"Xema.Builder.atom/1","doc":"Returns a tuple of `:atom` and the given keyword list.","ref":"Xema.Builder.html#atom/1"},{"type":"function","title":"Examples - Xema.Builder.atom/1","doc":"iex> Xema.Builder.atom(key: 42)\n    {:atom, key: 42}","ref":"Xema.Builder.html#atom/1-examples"},{"type":"function","title":"Xema.Builder.boolean/0","doc":"Returns :boolean.","ref":"Xema.Builder.html#boolean/0"},{"type":"function","title":"Xema.Builder.boolean/1","doc":"Returns a tuple of `:boolean` and the given keyword list.","ref":"Xema.Builder.html#boolean/1"},{"type":"function","title":"Examples - Xema.Builder.boolean/1","doc":"iex> Xema.Builder.boolean(key: 42)\n    {:boolean, key: 42}","ref":"Xema.Builder.html#boolean/1-examples"},{"type":"function","title":"Xema.Builder.field/3","doc":"Specifies a field. This function will be used inside `xema/0`.\n\nArguments:\n\n+ `name`: the name of the field.\n+ `type`: the type of the field. The `type` can also be a `struct` or another\n   schema.\n+ `opts`: the rules for the field.","ref":"Xema.Builder.html#field/3"},{"type":"function","title":"Examples - Xema.Builder.field/3","doc":"iex> defmodule User do\n    ...>   use Xema\n    ...>\n    ...>   xema_struct do\n    ...>     field :name, :string, min_length: 1\n    ...>   end\n    ...> end\n    ...>\n    iex> %{\"name\" => \"Tim\"} |> User.cast!() |> Map.from_struct()\n    %{name: \"Tim\"}\n\nFor more examples see \"[Examples: Struct](examples.html#struct)\".","ref":"Xema.Builder.html#field/3-examples"},{"type":"function","title":"Xema.Builder.float/0","doc":"Returns :float.","ref":"Xema.Builder.html#float/0"},{"type":"function","title":"Xema.Builder.float/1","doc":"Returns a tuple of `:float` and the given keyword list.","ref":"Xema.Builder.html#float/1"},{"type":"function","title":"Examples - Xema.Builder.float/1","doc":"iex> Xema.Builder.float(key: 42)\n    {:float, key: 42}","ref":"Xema.Builder.html#float/1-examples"},{"type":"function","title":"Xema.Builder.integer/0","doc":"Returns :integer.","ref":"Xema.Builder.html#integer/0"},{"type":"function","title":"Xema.Builder.integer/1","doc":"Returns a tuple of `:integer` and the given keyword list.","ref":"Xema.Builder.html#integer/1"},{"type":"function","title":"Examples - Xema.Builder.integer/1","doc":"iex> Xema.Builder.integer(key: 42)\n    {:integer, key: 42}","ref":"Xema.Builder.html#integer/1-examples"},{"type":"function","title":"Xema.Builder.keyword/0","doc":"Returns :keyword.","ref":"Xema.Builder.html#keyword/0"},{"type":"function","title":"Xema.Builder.keyword/1","doc":"Returns a tuple of `:keyword` and the given keyword list.","ref":"Xema.Builder.html#keyword/1"},{"type":"function","title":"Examples - Xema.Builder.keyword/1","doc":"iex> Xema.Builder.keyword(key: 42)\n    {:keyword, key: 42}","ref":"Xema.Builder.html#keyword/1-examples"},{"type":"function","title":"Xema.Builder.list/0","doc":"Returns :list.","ref":"Xema.Builder.html#list/0"},{"type":"function","title":"Xema.Builder.list/1","doc":"Returns a tuple of `:list` and the given keyword list.","ref":"Xema.Builder.html#list/1"},{"type":"function","title":"Examples - Xema.Builder.list/1","doc":"iex> Xema.Builder.list(key: 42)\n    {:list, key: 42}","ref":"Xema.Builder.html#list/1-examples"},{"type":"function","title":"Xema.Builder.map/0","doc":"Returns :map.","ref":"Xema.Builder.html#map/0"},{"type":"function","title":"Xema.Builder.map/1","doc":"Returns a tuple of `:map` and the given keyword list.","ref":"Xema.Builder.html#map/1"},{"type":"function","title":"Examples - Xema.Builder.map/1","doc":"iex> Xema.Builder.map(key: 42)\n    {:map, key: 42}","ref":"Xema.Builder.html#map/1-examples"},{"type":"function","title":"Xema.Builder.number/0","doc":"Returns :number.","ref":"Xema.Builder.html#number/0"},{"type":"function","title":"Xema.Builder.number/1","doc":"Returns a tuple of `:number` and the given keyword list.","ref":"Xema.Builder.html#number/1"},{"type":"function","title":"Examples - Xema.Builder.number/1","doc":"iex> Xema.Builder.number(key: 42)\n    {:number, key: 42}","ref":"Xema.Builder.html#number/1-examples"},{"type":"function","title":"Xema.Builder.one_of/2","doc":"Returns a tuple with the given `type` (default `:any`) and the given schemas\ntagged by the keyword `:one_of`. This function provides a shortcut for\nsomething like `integer(one_of: [...])` or `any(one_of: [...])`.","ref":"Xema.Builder.html#one_of/2"},{"type":"function","title":"Examples - Xema.Builder.one_of/2","doc":"iex> Xema.Builder.one_of([:integer, :string])\n    {:any, one_of: [:integer, :string] }\n\n    iex> Xema.Builder.one_of(:integer, [[minimum: 10], [maximum: 5]])\n    {:integer, one_of: [[minimum: 10], [maximum: 5]]}\n\n```elixir\ndefmodule MySchema do\n  use Xema\n\n  xema do\n    one_of [\n      list(items: integer(minimum: 1, maximum: 66)),\n      list(items: integer(minimum: 33, maximum: 100))\n    ]\n  end\nend\n\nMySchema.valid?([20, 30]) #=> true\nMySchema.valid?([40, 50]) #=> false\nMySchema.valid?([60, 70]) #=> true\nMySchema.valid?([10, 90]) #=> false\n```","ref":"Xema.Builder.html#one_of/2-examples"},{"type":"function","title":"Xema.Builder.ref/1","doc":"Returns the tuple `{:ref, ref}`.","ref":"Xema.Builder.html#ref/1"},{"type":"function","title":"Xema.Builder.required/1","doc":"Sets the list of required fields.  Specifies a field. This function will be\nused inside `xema/0`.","ref":"Xema.Builder.html#required/1"},{"type":"function","title":"Examples - Xema.Builder.required/1","doc":"iex> defmodule Person do\n    ...>   use Xema\n    ...>\n    ...>   xema_struct do\n    ...>     field :name, :string, min_length: 1\n    ...>     required [:name]\n    ...>   end\n    ...> end\n    ...>\n    iex> %{\"name\" => \"Tim\"} |> Person.cast!() |> Map.from_struct()\n    %{name: \"Tim\"}","ref":"Xema.Builder.html#required/1-examples"},{"type":"function","title":"Xema.Builder.string/0","doc":"Returns :string.","ref":"Xema.Builder.html#string/0"},{"type":"function","title":"Xema.Builder.string/1","doc":"Returns a tuple of `:string` and the given keyword list.","ref":"Xema.Builder.html#string/1"},{"type":"function","title":"Examples - Xema.Builder.string/1","doc":"iex> Xema.Builder.string(key: 42)\n    {:string, key: 42}","ref":"Xema.Builder.html#string/1-examples"},{"type":"function","title":"Xema.Builder.strux/0","doc":"Returns `:struct`.","ref":"Xema.Builder.html#strux/0"},{"type":"function","title":"Xema.Builder.strux/1","doc":"Returns a tuple of `:struct` and the given keyword list when the function gets\na keyword list.\n\nReturns the tuple `{:struct, module: module}` when the function gets an atom.","ref":"Xema.Builder.html#strux/1"},{"type":"function","title":"Xema.Builder.strux/2","doc":"","ref":"Xema.Builder.html#strux/2"},{"type":"function","title":"Xema.Builder.tuple/0","doc":"Returns :tuple.","ref":"Xema.Builder.html#tuple/0"},{"type":"function","title":"Xema.Builder.tuple/1","doc":"Returns a tuple of `:tuple` and the given keyword list.","ref":"Xema.Builder.html#tuple/1"},{"type":"function","title":"Examples - Xema.Builder.tuple/1","doc":"iex> Xema.Builder.tuple(key: 42)\n    {:tuple, key: 42}","ref":"Xema.Builder.html#tuple/1-examples"},{"type":"macro","title":"Xema.Builder.xema/1","doc":"Creates a `schema`.","ref":"Xema.Builder.html#xema/1"},{"type":"macro","title":"Xema.Builder.xema/2","doc":"Creates a `schema` with the given name.","ref":"Xema.Builder.html#xema/2"},{"type":"macro","title":"Xema.Builder.xema_struct/1","doc":"Creates a `schema` and defines the corresponding struct.","ref":"Xema.Builder.html#xema_struct/1"},{"type":"protocol","title":"Xema.Castable","doc":"Converts data using the specified schema.","ref":"Xema.Castable.html"},{"type":"function","title":"Xema.Castable.cast/2","doc":"Converts the given data using the specified schema.","ref":"Xema.Castable.html#cast/2"},{"type":"type","title":"Xema.Castable.t/0","doc":"All the types that implement this protocol.","ref":"Xema.Castable.html#t:t/0"},{"type":"behaviour","title":"Xema.Caster","doc":"A behaviour for a caster.","ref":"Xema.Caster.html"},{"type":"callback","title":"Xema.Caster.cast/1","doc":"A callback for a caster.","ref":"Xema.Caster.html#c:cast/1"},{"type":"module","title":"Xema.Format","doc":"This module contains semantic validators for strings.","ref":"Xema.Format.html"},{"type":"function","title":"Xema.Format.date?/1","doc":"Checks if the `string` is a valid date representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 3339](https://tools.ietf.org/html/rfc3339), `false` otherwise.","ref":"Xema.Format.html#date?/1"},{"type":"function","title":"Xema.Format.date_time?/1","doc":"Checks if the `string` is a valid date time representation.\n\nThis function returns `true` if the value is a `string` and is formatted as\ndefined by [RFC 3339](https://tools.ietf.org/html/rfc3339), `false` otherwise.","ref":"Xema.Format.html#date_time?/1"},{"type":"function","title":"Xema.Format.email?/1","doc":"Checks if the `string` is a valid email representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 5322](https://tools.ietf.org/html/rfc5322), `false` otherwise.\n\nThe regular expression was taken from\n[https://emailregex.com/](https://emailregex.com/).","ref":"Xema.Format.html#email?/1"},{"type":"function","title":"Examples - Xema.Format.email?/1","doc":"iex> import Xema.Format\n    ...> email?(\"marin.musterman@germany.net\")\n    true\n    ...> email?(\"Otto.Normalverbraucher\")\n    false\n    ...> email?(\"Otto.Normal@Verbraucher.NET\")\n    true\n    ...> email?(\"müller.aßa@foo.de\")\n    true\n    ...> email?(\"réñe@foo.de\")\n    true","ref":"Xema.Format.html#email?/1-examples"},{"type":"type","title":"Xema.Format.format/0","doc":"The list of supported validators.","ref":"Xema.Format.html#t:format/0"},{"type":"function","title":"Xema.Format.host?/1","doc":"Checks if the `string` is a valid host representation.\n\nThis function returns `true` if the value is a valid IPv4 address, IPv6\naddress, or a valid hostname, `false` otherwise.","ref":"Xema.Format.html#host?/1"},{"type":"function","title":"Examples - Xema.Format.host?/1","doc":"iex> import Xema.Format\n    iex>\n    iex> host?(\"127.0.0.1\")\n    true\n    iex> host?(\"localhost\")\n    true\n    iex> host?(\"elixirforum.com\")\n    true\n    iex> host?(\"go go go\")\n    false","ref":"Xema.Format.html#host?/1-examples"},{"type":"function","title":"Xema.Format.hostname?/1","doc":"Checks if the `string` is a valid hostname representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 1034](https://tools.ietf.org/html/rfc1034), `false` otherwise.","ref":"Xema.Format.html#hostname?/1"},{"type":"function","title":"Xema.Format.ipv4?/1","doc":"Checks if the `string` is a valid IPv4 address representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 2673](https://tools.ietf.org/html/rfc2673), `false` otherwise.","ref":"Xema.Format.html#ipv4?/1"},{"type":"function","title":"Xema.Format.ipv6?/1","doc":"Checks if the `string` is a valid IPv6 address representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 2373](https://tools.ietf.org/html/rfc2373), `false` otherwise.","ref":"Xema.Format.html#ipv6?/1"},{"type":"function","title":"Xema.Format.is?/2","doc":"Checks if the value matches the given type. The function expected a available\n`format` and a `string` to check. Returns `true` for a valid `string`, `false`\notherwise.","ref":"Xema.Format.html#is?/2"},{"type":"function","title":"Examples - Xema.Format.is?/2","doc":"iex> Xema.Format.is?(:email, \"foo@bar.net\")\n    true\n    iex> Xema.Format.is?(:email, \"foo.bar.net\")\n    false","ref":"Xema.Format.html#is?/2-examples"},{"type":"function","title":"Xema.Format.json_pointer?/1","doc":"Checks if the `string` is a valid JSON pointer representation.","ref":"Xema.Format.html#json_pointer?/1"},{"type":"function","title":"Xema.Format.regex?/1","doc":"Return true if `string` contains a regular expression.","ref":"Xema.Format.html#regex?/1"},{"type":"function","title":"Xema.Format.relative_json_pointer?/1","doc":"Checks if the `string` is a valid JSON pointer representation.","ref":"Xema.Format.html#relative_json_pointer?/1"},{"type":"macro","title":"Xema.Format.supports/1","doc":"Returns `true` for a valid format type, `false` otherwise.  This macro can be\nused in guards.","ref":"Xema.Format.html#supports/1"},{"type":"function","title":"Xema.Format.time?/1","doc":"Checks if the `string` is a valid time representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 3339](https://tools.ietf.org/html/rfc3339), `false` otherwise.","ref":"Xema.Format.html#time?/1"},{"type":"function","title":"Xema.Format.uri?/1","doc":"Checks if the `string` is a valid URI representation.\n\nThis function returns `true` if the value is a string and is formatted as\ndefined by [RFC 3986](https://tools.ietf.org/html/rfc3986), `false` otherwise.\n\nThe following are two example URIs and their component parts:\n```code\n                      hierarchical part\n                            |\n        |-----------------------------------------|\n                    authority               path\n                        |                    |\n        |-------------------------------||--------|\n  abc://username:password@example.com:123/path/data?key=value#fragid1\n  |-|   |---------------| |---------| |-|           |-------| |-----|\n   |            |              |       |                |        |\nscheme  user information     host     port            query   fragment\n\n  urn:example:mammal:monotreme:echidna\n  |-| |------------------------------|\n   |                 |\nscheme              path\n```\nWikipedia: [Uniform Resource Identifier](https://en.wikipedia.org/wiki/Uniform_Resource_Identifier)","ref":"Xema.Format.html#uri?/1"},{"type":"function","title":"Xema.Format.uri_fragment?/1","doc":"Checks if the `string` is a valid URI fragment representation.","ref":"Xema.Format.html#uri_fragment?/1"},{"type":"function","title":"Xema.Format.uri_path?/1","doc":"Checks if the `string` is a valid URI path representation.\n\nSee also `Xema.Format.uri?/1`.","ref":"Xema.Format.html#uri_path?/1"},{"type":"function","title":"Xema.Format.uri_query?/1","doc":"Checks if the `string` is a valid URI query representation.","ref":"Xema.Format.html#uri_query?/1"},{"type":"function","title":"Xema.Format.uri_reference?/1","doc":"Checks if the `string` is a valid URI reference representation.","ref":"Xema.Format.html#uri_reference?/1"},{"type":"function","title":"Xema.Format.uri_template?/1","doc":"Checks if the `string` is a valid URI template representation.","ref":"Xema.Format.html#uri_template?/1"},{"type":"function","title":"Xema.Format.uri_template_path?/1","doc":"Checks if the `string` is a valid URI template path representation.","ref":"Xema.Format.html#uri_template_path?/1"},{"type":"function","title":"Xema.Format.uri_userinfo?/1","doc":"Checks if the `string` is a valid URI user info.\n\nSee also `Xema.Format.uri?/1`.","ref":"Xema.Format.html#uri_userinfo?/1"},{"type":"module","title":"Xema.JsonSchema","doc":"Converts a JSON Schema to Xema source.","ref":"Xema.JsonSchema.html"},{"type":"type","title":"Xema.JsonSchema.json_schema/0","doc":"","ref":"Xema.JsonSchema.html#t:json_schema/0"},{"type":"type","title":"Xema.JsonSchema.opts/0","doc":"","ref":"Xema.JsonSchema.html#t:opts/0"},{"type":"function","title":"Xema.JsonSchema.to_xema/2","doc":"This function converts a JSON Schema in Xema schema source. The argument\n`json_schema` is expected as a decoded JSON Schema.\n\nAll keys that are not standard JSON Schema keywords have to be known atoms. If\nthe schema has additional keys that are unknown atoms the option\n`atom: :force` is needed. In this case the atoms will be created. This is not\nneeded for keys expected by JSON Schema (e.g. in properties)\n\nOptions:\n* `:draft` specifies the draft to check the given JSON Schema. Possible values\n  are `\"draft4\"`, `\"draft6\"`, and `\"draft7\"`, default is `\"draft7\"`. If\n  `:draft` not set and the schema contains `$schema` then the value for\n  `$schema` is used for this option.\n* `:atom` creates atoms for unknown atoms when set to `:force`. This is just\n  needed for additional JSON Schema keywords.","ref":"Xema.JsonSchema.html#to_xema/2"},{"type":"function","title":"Examples - Xema.JsonSchema.to_xema/2","doc":"iex> Xema.JsonSchema.to_xema(%{\"type\" => \"integer\", \"minimum\" => 5})\n    {:integer, [minimum: 5]}\n\n    iex> schema = %{\n    ...>   \"type\" => \"object\",\n    ...>   \"properties\" => %{\"foo\" => %{\"type\" => \"integer\"}}\n    ...> }\n    iex> Xema.JsonSchema.to_xema(schema)\n    {:map, [keys: :strings, properties: %{\"foo\" => :integer}]}\n\n    iex> Xema.JsonSchema.to_xema(%{\"type\" => \"integer\", \"foo\" => \"bar\"}, atom: :force)\n    {:integer, [foo: \"bar\"]}","ref":"Xema.JsonSchema.html#to_xema/2-examples"},{"type":"behaviour","title":"Xema.Loader","doc":"The behaviour for loaders.\n\nFor the loader configuration see \"[Configure a loader](loader.html)\".","ref":"Xema.Loader.html"},{"type":"function","title":"Xema.Loader.fetch/1","doc":"","ref":"Xema.Loader.html#fetch/1"},{"type":"callback","title":"Xema.Loader.fetch/1","doc":"This function expected an URI, to fetch the required data to create a schema.","ref":"Xema.Loader.html#c:fetch/1"},{"type":"module","title":"Xema.NoLoader","doc":"The default loader.\n\nFor the loader configuration see \"[Configure a loader](loader.html)\".","ref":"Xema.NoLoader.html"},{"type":"function","title":"Xema.NoLoader.fetch/1","doc":"Returns for `any` data always the error tuple\n`{:error, \"No loader configured.\"}`.","ref":"Xema.NoLoader.html#fetch/1"},{"type":"function","title":"Examples - Xema.NoLoader.fetch/1","doc":"iex> Xema.NoLoader.fetch(nil)\n    {:error, \"No loader configured.\"}","ref":"Xema.NoLoader.html#fetch/1-examples"},{"type":"module","title":"Xema.Ref","doc":"This module contains a struct and functions to represent and handle\nreferences.","ref":"Xema.Ref.html"},{"type":"function","title":"Xema.Ref.fetch!/3","doc":"Returns the schema and the root for the given `ref` and `xema`.","ref":"Xema.Ref.html#fetch!/3"},{"type":"function","title":"Xema.Ref.fragment/1","doc":"","ref":"Xema.Ref.html#fragment/1"},{"type":"function","title":"Xema.Ref.key/1","doc":"Returns the reference key for a `Ref` or an `URI`.","ref":"Xema.Ref.html#key/1"},{"type":"function","title":"Xema.Ref.new/1","doc":"Creates a new reference from the given `pointer`.","ref":"Xema.Ref.html#new/1"},{"type":"function","title":"Xema.Ref.new/2","doc":"Creates a new reference from the given `pointer` and `uri`.","ref":"Xema.Ref.html#new/2"},{"type":"type","title":"Xema.Ref.t/0","doc":"A reference contains a `pointer` and an optional `uri`.","ref":"Xema.Ref.html#t:t/0"},{"type":"function","title":"Xema.Ref.validate/3","doc":"Validates the given value with the referenced schema.","ref":"Xema.Ref.html#validate/3"},{"type":"module","title":"Xema.Schema","doc":"This module contains the struct for the keywords of a schema.","ref":"Xema.Schema.html"},{"type":"function","title":"Xema.Schema.fetch/2","doc":"Fetches a subschema from the `schema` by the given `pointer`.\n\nIf `schema` contains the given pointer with a subschema, then `{:ok, schema}`\nis returned otherwise `:error`.","ref":"Xema.Schema.html#fetch/2"},{"type":"function","title":"Xema.Schema.fetch!/2","doc":"Fetches a subschema from the `schema` by the given `pointer`.\n\nIf `schema` contains the given pointer with a subschema, then `{:ok, schema}`\nis returned otherwise a `SchemaError` is raised.","ref":"Xema.Schema.html#fetch!/2"},{"type":"function","title":"Xema.Schema.keywords/0","doc":"Returns all keywords in a list.\n\nThe key `:data` is not a regular keyword and is not in the list.","ref":"Xema.Schema.html#keywords/0"},{"type":"function","title":"Xema.Schema.new/1","doc":"Returns a `%Schema{}` for the given `keywords` in the keyword list.","ref":"Xema.Schema.html#new/1"},{"type":"type","title":"Xema.Schema.t/0","doc":"The struct contains the keywords for a schema.\n\n* `additional_items` disallow additional items, if set to false. The keyword\n  can also contain a schema to specify the type of additional items.\n* `additional_properties` disallow additional properties, if set to true.\n* 'all_of' a list of schemas they must all be valid.\n* 'any_of' a list of schemas with any valid schema.\n* `caster` a custom caster. This can be a function, a tuple with\n  module and function name, or a `Xema.Caster` behaviour.\n* `comment` for the schema.\n* `const` specifies a constant.\n* `content_encoding` annotation for the encoding.\n* `content_media_type` annotation for the media type.\n* `contains` validates a list whether any item is valid for the given schema.\n* `data` none schema data. Values in `data` will be interpreted as schemas\n  when possible. It is not recommended to put any data or schemas under this\n  key. The `data` property is mainly for compatibility with JsonSchema.\n* `default` this keyword can be used to supply a default value for JSON and\n  `defstruct`.\n* `definitions` contains schemas for reuse.\n* `dependencies` allows the schema of the map to change based on the presence\n  of certain special properties\n* `description` of the schema.\n* `else` see `if`, `then`, `else`.\n* `enum` specifies an enumeration\n* `examples` the value of this keyword must be an array. There are no\n  restrictions placed on the values within the array.\n* `exclusive_maximum` is a boolean. When true, it indicates that the range\n  excludes the maximum value.\n* `exclusive_minimum` is a boolean. When true, it indicates that the range\n  excludes the minimum value.\n* `format` semantic validation.\n* `id` a unique identifier.\n* `if`, `then`, `else`: These keywords work together to implement conditional\n  application of a subschema based on the outcome of another subschema.\n* `items` specifies the type(s) of the items.\n* `keys` could be `:atoms` or `:strings`.\n* `max_items` the maximum length of list.\n* `max_length` the maximum length of string.\n* `max_properties` the maximum count of properties for the map.\n* `maximum` the maximum value.\n* `min_items` the minimal length of list.\n* `min_length` the minimal length of string.\n* `min_properties` the minimal count of properties for the map.\n* `minimum` the minimum value.\n* `module` the module of a struct.\n* `multiple_of` is a number greater 0. The value has to be a multiple of this\n  number.\n* `not` negates the given schema\n* `one_of` the given data must be valid against exactly one of the given\n  subschemas.\n* `pattern_properties` specifies schemas for properties by patterns\n* `pattern` restrict a string to a particular regular expression.\n* `properties` specifies schemas for properties.\n* `property_names` a schema to check property names.\n* `ref` a reference to a schema.\n* `required` contains a set of required properties.\n* `schema` declares the used schema.\n* `title` of the schema.\n* `then` see `if`, `then`, `else`\n* `type` specifies the data type for a schema.\n* `unique_items` disallow duplicate items, if set to true.\n* `validator` a custom validator. This can be a function, a tuple with\n  module and function name, or a `Xema.Validator` behaviour.","ref":"Xema.Schema.html#t:t/0"},{"type":"function","title":"Xema.Schema.to_map/1","doc":"Returns the `%Schema{}` as a map. Items which a `nil` value are not in the\nmap.","ref":"Xema.Schema.html#to_map/1"},{"type":"type","title":"Xema.Schema.type/0","doc":"The `type` for the schema.","ref":"Xema.Schema.html#t:type/0"},{"type":"function","title":"Xema.Schema.types/0","doc":"Returns all available `type`s in a list.","ref":"Xema.Schema.html#types/0"},{"type":"type","title":"Xema.Schema.xema/0","doc":"","ref":"Xema.Schema.html#t:xema/0"},{"type":"module","title":"Xema.SchemaValidator","doc":"A validator for the schema input.","ref":"Xema.SchemaValidator.html"},{"type":"function","title":"Xema.SchemaValidator.validate!/1","doc":"Returns `:ok` if the data valid against the xema schema.","ref":"Xema.SchemaValidator.html#validate!/1"},{"type":"module","title":"Xema.Utils","doc":"Some utilities for Xema.","ref":"Xema.Utils.html"},{"type":"function","title":"Xema.Utils.has_key?/2","doc":"Returns whether the given `key` exists in the given `value`.\n\nReturns true if\n* `value` is a map and contains `key` as a key.\n* `value` is a keyword and contains `key` as a key.\n* `value` is a list of tuples with `key`as the first element.","ref":"Xema.Utils.html#has_key?/2"},{"type":"function","title":"Examples - Xema.Utils.has_key?/2","doc":"iex> alias Xema.Utils\n      iex> Utils.has_key?(%{foo: 5}, :foo)\n      true\n      iex> Utils.has_key?([foo: 5], :foo)\n      true\n      iex> Utils.has_key?([{\"foo\", 5}], \"foo\")\n      true\n      iex> Utils.has_key?([{\"foo\", 5}], \"bar\")\n      false\n      iex> Utils.has_key?([], \"bar\")\n      false","ref":"Xema.Utils.html#has_key?/2-examples"},{"type":"function","title":"Xema.Utils.size/1","doc":"Returns the size of a `list` or `tuple`.","ref":"Xema.Utils.html#size/1"},{"type":"function","title":"Xema.Utils.to_existing_atom/1","doc":"Converts the given `string` to an existing atom. Returns `nil` if the\natom does not exist.","ref":"Xema.Utils.html#to_existing_atom/1"},{"type":"function","title":"Examples - Xema.Utils.to_existing_atom/1","doc":"iex> import Xema.Utils\n      iex> to_existing_atom(:my_atom)\n      :my_atom\n      iex> to_existing_atom(\"my_atom\")\n      :my_atom\n      iex> to_existing_atom(\"not_existing_atom\")\n      nil","ref":"Xema.Utils.html#to_existing_atom/1-examples"},{"type":"function","title":"Xema.Utils.to_sorted_list/2","doc":"Converts a `map` with integer keys or integer keys represented as strings to\na sorted list.\n\nReturns an ok tuple with the list or an `:error` atom.","ref":"Xema.Utils.html#to_sorted_list/2"},{"type":"function","title":"Options: - Xema.Utils.to_sorted_list/2","doc":"* `keys` - if `true` the resulting list has `{key, value}` items, with false\n             the list contains the values. Defaults to `[keys: true]`.","ref":"Xema.Utils.html#to_sorted_list/2-options"},{"type":"function","title":"Examples - Xema.Utils.to_sorted_list/2","doc":"iex> alias Xema.Utils\n      iex> Utils.to_sorted_list(%{2 => \"b\", 3 => \"c\", 1 => \"a\"})\n      {:ok, [{1, \"a\"}, {2, \"b\"}, {3, \"c\"}]}\n      iex> Utils.to_sorted_list(%{\"2\" => \"b\", \"3\" => \"c\", \"1\" => \"a\"})\n      {:ok, [{\"1\", \"a\"}, {\"2\", \"b\"}, {\"3\", \"c\"}]}\n      iex> Utils.to_sorted_list(%{\"2\" => \"b\", \"x\" => \"c\", \"1\" => \"a\"})\n      :error\n      iex> Utils.to_sorted_list(%{\"2\" => \"b\", \"3\" => \"c\", \"1\" => \"a\"}, keys: true)\n      {:ok, [{\"1\", \"a\"}, {\"2\", \"b\"}, {\"3\", \"c\"}]}\n      iex> Utils.to_sorted_list(%{\"2\" => \"b\", \"3\" => \"c\", \"1\" => \"a\"}, keys: false)\n      {:ok, [\"a\", \"b\", \"c\"]}","ref":"Xema.Utils.html#to_sorted_list/2-examples"},{"type":"function","title":"Xema.Utils.update_uri/2","doc":"Returns `nil` if `uri_1` and `uri_2` are `nil`.\nParses a URI when the other URI is `nil`.\nMerges URIs if both are not nil.","ref":"Xema.Utils.html#update_uri/2"},{"type":"behaviour","title":"Xema.Validator","doc":"This module contains all validators to check data against a schema.","ref":"Xema.Validator.html"},{"type":"type","title":"Xema.Validator.result/0","doc":"","ref":"Xema.Validator.html#t:result/0"},{"type":"callback","title":"Xema.Validator.validate/1","doc":"A callback for custom validators. For an example see:\n[Custom validators](examples.html#custom-validator)","ref":"Xema.Validator.html#c:validate/1"},{"type":"function","title":"Xema.Validator.validate/2","doc":"Validates `data` against the given `schema`.","ref":"Xema.Validator.html#validate/2"},{"type":"exception","title":"Xema.CastError","doc":"Raised when a cast fails.","ref":"Xema.CastError.html"},{"type":"function","title":"Xema.CastError.format_error/1","doc":"Formats the error map to an error message.","ref":"Xema.CastError.html#format_error/1"},{"type":"type","title":"Xema.CastError.t/0","doc":"","ref":"Xema.CastError.html#t:t/0"},{"type":"exception","title":"Xema.SchemaError","doc":"Raised when a schema can't be build.","ref":"Xema.SchemaError.html"},{"type":"exception","title":"Xema.ValidationError","doc":"Raised when a validation fails.","ref":"Xema.ValidationError.html"},{"type":"function","title":"Xema.ValidationError.format_error/1","doc":"This function returns an error message for an error or error tuple.","ref":"Xema.ValidationError.html#format_error/1"},{"type":"function","title":"Example - Xema.ValidationError.format_error/1","doc":"```elixir\niex> schema = Xema.new(:integer)\n...> schema\n...>   |> Xema.Validator.validate(1.1)\n...>   |> Xema.ValidationError.format_error()\n\"Expected :integer, got 1.1.\"\niex> schema\n...>   |> Xema.validate(1.1)\n...>   |> Xema.ValidationError.format_error()\n\"Expected :integer, got 1.1.\"\n```","ref":"Xema.ValidationError.html#format_error/1-example"},{"type":"type","title":"Xema.ValidationError.opts/0","doc":"","ref":"Xema.ValidationError.html#t:opts/0"},{"type":"type","title":"Xema.ValidationError.path/0","doc":"","ref":"Xema.ValidationError.html#t:path/0"},{"type":"type","title":"Xema.ValidationError.t/0","doc":"","ref":"Xema.ValidationError.html#t:t/0"},{"type":"function","title":"Xema.ValidationError.travers_errors/4","doc":"Traverse the error tree and invokes the given function.","ref":"Xema.ValidationError.html#travers_errors/4"},{"type":"function","title":"Example - Xema.ValidationError.travers_errors/4","doc":"```elixir\niex> fun = fn _error, path, acc ->\n...>   [\"Error at \" <> inspect(path) | acc]\n...> end\niex>\niex> schema = Xema.new(\n...>   properties: %{\n...>     int: :integer,\n...>     names: {:list, items: :string},\n...>     num: [any_of: [:integer, :float]]\n...>   }\n...> )\niex>\niex> data = %{int: \"x\", names: [1, \"x\", 5], num: :foo}\niex>\niex> schema\n...>   |> Xema.Validator.validate(data)\n...>   |> Xema.ValidationError.travers_errors([], fun)\n[\n  \"Error at [:num]\",\n  \"Error at [:names, 2]\",\n  \"Error at [:names, 0]\",\n  \"Error at [:names]\",\n  \"Error at [:int]\",\n  \"Error at []\"\n]\n```","ref":"Xema.ValidationError.html#travers_errors/4-example"},{"type":"extras","title":"Xema","doc":"# Xema\n\nXema is a schema validator inspired by [JSON Schema](http://json-schema.org).\nXema supports the features documented in draft 04, 06, and 07 of the JSON-Schema\nspecification.\n\nXema allows you to annotate and validate elixir data structures.\n\nXema is in beta. If you try it and has an issue, report them.","ref":"readme.html"},{"type":"extras","title":"Usage","doc":"# Usage\n\nThis page describes all keywords that are available for creating a schema.\nAll schemas will construct in the \"raw\" format. The `Xema.Builder` provides some\nconvenience functions to create schemas. The recommended method to construct a\nschema is `use Xema` described in the module documentation of `Xema`.","ref":"usage.html"},{"type":"extras","title":"Type any - Usage","doc":"The schema any will accept any data.\n\n```elixir\niex> schema = Xema.new :any\niex> Xema.validate schema, 42\n:ok\niex> Xema.validate schema, \"foo\"\n:ok\niex> Xema.validate schema, nil\n:ok\n```","ref":"usage.html#type-any"},{"type":"extras","title":"Type nil - Usage","doc":"The nil type matches only `nil`.\n\n```elixir\niex> schema = Xema.new :nil\niex> Xema.validate schema, nil\n:ok\niex> {:error, error} = Xema.validate schema, 0\n{:error, %Xema.ValidationError{\n  reason: %{type: :nil, value: 0}\n}}\niex> Exception.message(error)\n\"Expected nil, got 0.\"\n```","ref":"usage.html#type-nil"},{"type":"extras","title":"Type boolean - Usage","doc":"The boolean type matches only `true` and `false`.\n```elixir\niex> schema = Xema.new :boolean\niex> Xema.validate schema, true\n:ok\niex> Xema.valid? schema, false\ntrue\niex> {:error, error} = Xema.validate schema, 0\n{:error, %Xema.ValidationError{\n  reason: %{type: :boolean, value: 0}\n}}\niex> Exception.message(error)\n\"Expected :boolean, got 0.\"\niex> Xema.valid? schema, nil\nfalse\n```","ref":"usage.html#type-boolean"},{"type":"extras","title":"Type atom - Usage","doc":"The atom type matches only atoms. Schemas of type atom match also the atoms\n`true`, `false`, and `nil`.\n```elixir\niex> schema = Xema.new :atom\niex> Xema.validate schema, :foo\n:ok\niex> Xema.valid? schema, \"foo\"\nfalse\niex> {:error, error} = Xema.validate schema, 0\n{:error, %Xema.ValidationError{\n  reason: %{type: :atom, value: 0}\n}}\niex> Exception.message(error)\n\"Expected :atom, got 0.\"\niex> Xema.valid? schema, nil\ntrue\niex> Xema.valid? schema, false\ntrue\n```","ref":"usage.html#type-atom"},{"type":"extras","title":"Type string - Usage","doc":"`JSON Schema Draft: 4/6/7`\n\nThe string type is used for strings.\n\n```elixir\niex> schema = Xema.new :string\niex> Xema.validate schema, \"José\"\n:ok\niex> {:error, error} = Xema.validate schema, 42\n{:error, %Xema.ValidationError{\n  reason: %{type: :string, value: 42}\n}}\niex> Exception.message(error)\n\"Expected :string, got 42.\"\niex> Xema.valid? schema, \"José\"\ntrue\niex> Xema.valid? schema, 42\nfalse\n```","ref":"usage.html#type-string"},{"type":"extras","title":"Length - Usage","doc":"The length of a string can be constrained using the `min_length` and `max_length`\nkeywords. For both keywords, the value must be a non-negative number.\n\n```elixir\niex> schema = Xema.new {:string, min_length: 2, max_length: 3}\niex> {:error, error} = Xema.validate schema, \"a\"\n{:error, %Xema.ValidationError{\n  reason: %{value: \"a\", min_length: 2}\n}}\niex> Exception.message(error)\n~s|Expected minimum length of 2, got \"a\".|\niex> Xema.validate schema, \"ab\"\n:ok\niex> Xema.validate schema, \"abc\"\n:ok\niex> {:error, error} = Xema.validate schema, \"abcd\"\n{:error, %Xema.ValidationError{\n  reason: %{value: \"abcd\", max_length: 3}\n}}\niex> Exception.message(error)\n~s|Expected maximum length of 3, got \"abcd\".|\n```","ref":"usage.html#length"},{"type":"extras","title":"Regular Expression - Usage","doc":"The `pattern` keyword is used to restrict a string to a particular regular\nexpression.\n\n```elixir\niex> schema = Xema.new {:string, pattern: ~r/[0-9]-[A-B]+/}\niex> Xema.validate schema, \"1-AB\"\n:ok\niex> {:error, error} = Xema.validate schema, \"foo\"\n{:error, %Xema.ValidationError{\n  reason: %{value: \"foo\", pattern: ~r/[0-9]-[A-B]+/}\n}}\niex> Exception.message(error)\n~s|Pattern ~r/[0-9]-[A-B]+/ does not match value \"foo\".|\n```\n\nThe regular expression can also be a string.\n\n```elixir\niex> schema = Xema.new {:string, pattern: \"[0-9]-[A-B]+\"}\niex> Xema.validate schema, \"1-AB\"\n:ok\niex> {:error, error} = Xema.validate schema, \"foo\"\n{:error, %Xema.ValidationError{\n  reason: %{value: \"foo\", pattern: ~r/[0-9]-[A-B]+/}\n}}\niex> Exception.message(error)\n~s|Pattern ~r/[0-9]-[A-B]+/ does not match value \"foo\".|\n```","ref":"usage.html#regular-expression"},{"type":"extras","title":"Format - Usage","doc":"`JSON Schema Draft: 4/6/7`\n\nBasic semantic validation of strings.\n\n* `:date_time` validation as defined by [RFC 3339](https://tools.ietf.org/html/rfc3339)\n```elixir\niex> schema = Xema.new {:string, format: :date_time}\niex> Xema.valid? schema, \"today\"\nfalse\niex> Xema.valid? schema, \"1963-06-19T08:30:06.283185Z\"\ntrue\n```\n\n* `:email` validation as defined by [RFC 5322](https://tools.ietf.org/html/rfc5322)\n```elixir\niex> {:string, format: :email}\n...> |> Xema.new()\n...> |> Xema.valid?(\"marion.mustermann@otto.net\")\ntrue\n```\n\n* `:host` checks if the `string` is an valid IPv4, IPv6, or hostname.\n* `:hostname` validation as defined by [RFC 1034](https://tools.ietf.org/html/rfc1034)\n* `:ipv4` validation as defined by [RFC 2673](https://tools.ietf.org/html/rfc2673)\n* `:ipv6` validation as defined by [RFC 2373](https://tools.ietf.org/html/rfc2373)\n* `:uri` validation as defindex by [RFC 3986](https://tools.ietf.org/html/rfc3986)\n  * `:uri_fragment`\n  * `:uri_path`\n  * `:uri_query`\n  * `:uri_userinfo`\n\n`JSON Schema Draft: -/-/7`\n\n* `:regex`checks if the `string` is a valid regular expression.","ref":"usage.html#format"},{"type":"extras","title":"Types number, integer and float - Usage","doc":"There are three numeric types in Xema: `number`, `integer` and `float`. They\nshare the same validation keywords.\n\nThe `number` type is used for numbers.\n```elixir\niex> schema = Xema.new :number\niex> Xema.validate schema, 42\n:ok\niex> Xema.validate schema, 21.5\n:ok\niex> {:error, error} = Xema.validate schema, \"foo\"\n{:error, %Xema.ValidationError{\n  reason: %{type: :number, value: \"foo\"}\n}}\niex> Exception.message(error)\n~s|Expected :number, got \"foo\".|\n```\n\nThe `integer` type is used for integral numbers.\n```elixir\niex> schema = Xema.new :integer\niex> Xema.validate schema, 42\n:ok\niex> {:error, error} = Xema.validate schema, 21.5\n{:error, %Xema.ValidationError{\n  reason: %{type: :integer, value: 21.5}\n}}\niex> Exception.message(error)\n\"Expected :integer, got 21.5.\"\n```\n\nThe `float` type is used for floating point numbers.\n```elixir\niex> schema = Xema.new :float\niex> {:error, error} = Xema.validate schema, 42\n{:error, %Xema.ValidationError{\n  reason: %{type: :float, value: 42}\n}}\niex> Exception.message(error)\n\"Expected :float, got 42.\"\niex> Xema.validate schema, 21.5\n:ok\n```","ref":"usage.html#types-number-integer-and-float"},{"type":"extras","title":"Multiples - Usage","doc":"`JSON Schema Draft: 4/6/7`\n\nNumbers can be restricted to a multiple of a given number, using the\n`multiple_of` keyword. It may be set to any positive number.\n\n```elixir\niex> schema = Xema.new {:number, multiple_of: 2}\niex> Xema.validate schema, 8\n:ok\niex> {:error, error} = Xema.validate schema, 7\n{:error, %Xema.ValidationError{\n  reason: %{value: 7, multiple_of: 2}\n}}\niex> Exception.message(error)\n\"Value 7 is not a multiple of 2.\"\niex> Xema.valid? schema, 8.0\ntrue\n```","ref":"usage.html#multiples"},{"type":"extras","title":"Range - Usage","doc":"`JSON Schema Draft: 4/-/-`\n\nRanges of numbers are specified using a combination of the `minimum`, `maximum`,\n`exclusive_minimum` and `exclusive_maximum` keywords.\n* `minimum` specifies a minimum numeric value.\n* `exclusive_minimum` is a boolean. When true, it indicates that the range\n   excludes the minimum value, i.e., x > minx > min. When false (or not included),\n   it indicates that the range includes the minimum value, i.e., x≥minx≥min.\n* `maximum` specifies a maximum numeric value.\n* `exclusive_maximum` is a boolean. When true, it indicates that the range\n   excludes the maximum value, i.e., x   schema = Xema.new {\n...>   :float,\n...>   minimum: 1.2, maximum: 1.4, exclusive_maximum: true\n...> }\niex> {:error, error} = Xema.validate schema, 1.1\n{:error, %Xema.ValidationError{\n  reason: %{value: 1.1, minimum: 1.2}\n}}\niex> Exception.message(error)\n\"Value 1.1 is less than minimum value of 1.2.\"\niex> Xema.validate schema, 1.2\n:ok\niex> Xema.valid? schema, 1.3\ntrue\niex> {:error, error} = Xema.validate schema, 1.4\n{:error, %Xema.ValidationError{\n  reason: %{value: 1.4, maximum: 1.4, exclusive_maximum: true}\n}}\niex> Exception.message(error)\n\"Value 1.4 equals exclusive maximum value of 1.4.\"\niex> {:error, error} = Xema.validate schema, 1.5\n{:error, %Xema.ValidationError{\n  reason: %{value: 1.5, maximum: 1.4, exclusive_maximum: true}\n}}\niex> Exception.message(error)\n\"Value 1.5 exceeds maximum value of 1.4.\"\n```\n\n`JSON Schema Draft: 6/7`\n\nThe keywords `exclusive_maximum` and `exclusive_minimum` changed from a boolean\nto a number. Wherever one of these would be `true` before, they have now the\nvalue of the corresponding keyword `maximum` or `minimum`. The keyword\n`maximum`/`minimum` can be removed.\n\n```elixir\niex> schema = Xema.new {:float, minimum: 1.2, exclusive_maximum: 1.4}\niex> {:error, error} = Xema.validate schema, 1.1\n{:error, %Xema.ValidationError{\n  reason: %{value: 1.1, minimum: 1.2}\n}}\niex> Exception.message(error)\n\"Value 1.1 is less than minimum value of 1.2.\"\niex> Xema.validate schema, 1.2\n:ok\niex> Xema.valid? schema, 1.3\ntrue\niex> {:error, error} = Xema.validate schema, 1.4\n{:error, %Xema.ValidationError{\n  reason: %{value: 1.4, exclusive_maximum: 1.4}\n}}\niex> Exception.message(error)\n\"Value 1.4 equals exclusive maximum value of 1.4.\"\niex> {:error, error} = Xema.validate schema, 1.5\n{:error, %Xema.ValidationError{\n  reason: %{value: 1.5, exclusive_maximum: 1.4}\n}}\niex> Exception.message(error)\n\"Value 1.5 exceeds maximum value of 1.4.\"\n```","ref":"usage.html#range"},{"type":"extras","title":"Type list - Usage","doc":"List are used for ordered elements, each element may be of a different type.\n\n```elixir\niex> schema = Xema.new :list\niex> Xema.valid? schema, [1, \"two\", 3.0]\ntrue\niex> {:error, error} = Xema.validate schema, 9\n{:error, %Xema.ValidationError{\n  reason: %{type: :list, value: 9}\n}}\niex> Exception.message(error)\n\"Expected :list, got 9.\"\n```","ref":"usage.html#type-list"},{"type":"extras","title":"Items - Usage","doc":"The `items` keyword will be used to validate all items of a list to a single\nschema.\n\n```elixir\niex> schema = Xema.new {:list, items: :string}\niex> Xema.valid? schema, [\"a\", \"b\", \"abc\"]\ntrue\niex> {:error, error} = Xema.validate schema, [\"a\", 1]\n{:error, %Xema.ValidationError{\n  reason: %{items: %{1 => %{type: :string, value: 1}}}\n}}\niex> Exception.message(error)\n\"Expected :string, got 1, at [1].\"\n```\n\nThe next example shows how to add keywords to the items schema.\n\n```elixir\niex> schema = Xema.new {:list, items: {:integer, minimum: 1, maximum: 10}}\niex> Xema.validate schema, [1, 2, 3]\n:ok\niex> {:error, error} = Xema.validate schema, [3, 2, 1, 0]\n{:error, %Xema.ValidationError{\n  reason: %{items: %{3 => %{value: 0, minimum: 1}}}\n}}\niex> Exception.message(error)\n\"Value 0 is less than minimum value of 1, at [3].\"\n```\n\n`items` can also be used to give each item a specific schema.\n\n```elixir\niex> schema = Xema.new {\n...>   :list,\n...>   items: [:integer, {:string, min_length: 5}]\n...> }\niex> Xema.valid? schema, [1, \"hello\"]\ntrue\niex> {:error, error} = Xema.validate schema, [1, \"five\"]\n{:error, %Xema.ValidationError{\n  reason: %{items: %{1 => %{value: \"five\", min_length: 5}}}\n}}\niex> Exception.message(error)\n~s|Expected minimum length of 5, got \"five\", at [1].|\n# It’s okay to not provide all of the items:\niex> Xema.validate schema, [1]\n:ok\n# And, by default, it’s also okay to add additional items to end:\niex> Xema.validate schema, [1, \"hello\", \"foo\"]\n:ok\n```","ref":"usage.html#items"},{"type":"extras","title":"Additional Items - Usage","doc":"The `additional_items` keyword controls whether it is valid to have additional\nitems in the array beyond what is defined in the schema.\n\n```elixir\niex> schema = Xema.new {\n...>   :list,\n...>   items: [:integer, {:string, min_length: 5}],\n...>   additional_items: false\n...> }\niex> Xema.validate schema, [1]\n:ok\n# It’s okay to not provide all of the items:\n# But, since additionalItems is false, we can’t provide extra items:\niex> {:error, error} = Xema.validate schema, [1, \"hello\", \"foo\"]\n{:error, %Xema.ValidationError{\n  reason: %{items: %{2 => %{additional_items: false}}}\n}}\niex> Exception.message(error)\n\"Unexpected additional item, at [2].\"\niex> {:error, error} = Xema.validate schema, [1, \"hello\", \"foo\", \"bar\"]\n{:error, %Xema.ValidationError{\n  reason: %{\n    items: %{\n      2 => %{additional_items: false},\n      3 => %{additional_items: false}\n    }\n  }\n}}\niex> Exception.message(error)\n\"\"\"\nUnexpected additional item, at [2].\nUnexpected additional item, at [3].\\\n\"\"\"\n```\n\nThe keyword can also contain a schema to specify the type of additional items.\n```elixir\niex> schema = Xema.new {\n...>   :list,\n...>   items: [:integer, {:string, min_length: 3}],\n...>   additional_items: :integer\n...> }\niex> Xema.valid? schema, [1, \"two\", 3, 4]\ntrue\niex> {:error, error} = Xema.validate schema, [1, \"two\", 3, \"four\"]\n{:error, %Xema.ValidationError{\n  reason: %{items: %{3 => %{type: :integer, value: \"four\"}}}\n}}\niex> Exception.message(error)\n~s|Expected :integer, got \"four\", at [3].|\n```","ref":"usage.html#additional-items"},{"type":"extras","title":"Length - Usage","doc":"The length of the array can be specified using the `min_items` and `max_items`\nkeywords. The value of each keyword must be a non-negative number.\n\n```elixir\niex> schema = Xema.new {:list, min_items: 2, max_items: 3}\niex> {:error, error} = Xema.validate schema, [1]\n{:error, %Xema.ValidationError{\n  reason: %{value: [1], min_items: 2}\n}}\niex> Exception.message(error)\n\"Expected at least 2 items, got [1].\"\niex> Xema.validate schema, [1, 2]\n:ok\niex> Xema.validate schema, [1, 2, 3]\n:ok\niex> {:error, error} = Xema.validate schema, [1, 2, 3, 4]\n{:error, %Xema.ValidationError{\n  reason: %{value: [1, 2, 3, 4], max_items: 3}\n}}\niex> Exception.message(error)\n\"Expected at most 3 items, got [1, 2, 3, 4].\"\n```","ref":"usage.html#length"},{"type":"extras","title":"Uniqueness - Usage","doc":"A schema can ensure that each of the items in an array is unique.\n\n```elixir\niex> schema = Xema.new {:list, unique_items: true}\niex> Xema.valid? schema, [1, 2, 3]\ntrue\niex> {:error, error} = Xema.validate schema, [1, 2, 3, 2, 1]\n{:error, %Xema.ValidationError{\n  reason: %{value: [1, 2, 3, 2, 1], unique_items: true}\n}}\niex> Exception.message(error)\n\"Expected unique items, got [1, 2, 3, 2, 1].\"\n```","ref":"usage.html#uniqueness"},{"type":"extras","title":"Type tuple - Usage","doc":"Tuples are intended as fixed-size containers for multiple elements. The\nvalidation of tuples is similar to lists.\n\n```elixir\niex> schema = Xema.new {:tuple, min_items: 2, max_items: 3}\niex> {:error, error} = Xema.validate schema, {1}\n{:error, %Xema.ValidationError{\n  reason: %{value: {1}, min_items: 2}\n}}\niex> Exception.message(error)\n\"Expected at least 2 items, got {1}.\"\niex> Xema.validate schema, {1, 2}\n:ok\niex> Xema.validate schema, {1, 2, 3}\n:ok\niex> {:error, error} = Xema.validate schema, {1, 2, 3, 4}\n{:error, %Xema.ValidationError{\n  reason: %{value: {1, 2, 3, 4}, max_items: 3}\n}}\niex> Exception.message(error)\n\"Expected at most 3 items, got {1, 2, 3, 4}.\"\n```","ref":"usage.html#type-tuple"},{"type":"extras","title":"Type map - Usage","doc":"Whenever you need a key-value store, maps are the “go to” data structure in\nElixir. Each of these pairs is conventionally referred to as a “property”.\n\n```elixir\niex> schema = Xema.new :map\niex> Xema.valid? schema, %{\"foo\" => \"bar\"}\ntrue\niex> {:error, error} = Xema.validate schema, \"bar\"\n{:error, %Xema.ValidationError{\n  reason: %{type: :map, value: \"bar\"}\n}}\niex> Exception.message(error)\n~s|Expected :map, got \"bar\".|\n# Using non-strings as keys are also valid:\niex> Xema.valid? schema, %{foo: \"bar\"}\ntrue\niex> Xema.valid? schema, %{1 => \"bar\"}\ntrue\n```","ref":"usage.html#type-map"},{"type":"extras","title":"Keys - Usage","doc":"The keyword `keys` can restrict the keys to atoms or strings.\n\nAtoms as keys:\n```elixir\niex> schema = Xema.new {:map, keys: :atoms}\niex> Xema.valid? schema, %{\"foo\" => \"bar\"}\nfalse\niex> Xema.valid? schema, %{foo: \"bar\"}\ntrue\niex> Xema.valid? schema, %{1 => \"bar\"}\nfalse\n```\n\nStrings as keys:\n```elixir\niex> schema = Xema.new {:map, keys: :strings}\niex> Xema.valid? schema, %{\"foo\" => \"bar\"}\ntrue\niex> Xema.valid? schema, %{foo: \"bar\"}\nfalse\niex> Xema.valid? schema, %{1 => \"bar\"}\nfalse\n```","ref":"usage.html#keys"},{"type":"extras","title":"Properties - Usage","doc":"The properties on a map are defined using the `properties` keyword. The value\nof properties is a map, where each key is the name of a property and each\nvalue is a schema used to validate that property.\n\n```elixir\niex> schema = Xema.new {:map,\n...>   properties: %{\n...>     a: :integer,\n...>     b: {:string, min_length: 5}\n...>   }\n...> }\niex> Xema.valid? schema, %{a: 5, b: \"hello\"}\ntrue\niex> {:error, error} = Xema.validate schema, %{a: 5, b: \"ups\"}\n{:error, %Xema.ValidationError{\n  reason: %{\n    properties: %{\n      b: %{value: \"ups\", min_length: 5}\n    }\n  }\n}}\niex> Exception.message(error)\n\"Expected minimum length of 5, got \\\"ups\\\", at [:b].\"\n# Additinonal properties are allowed by default:\niex> Xema.valid? schema, %{a: 5, b: \"hello\", add: :prop}\ntrue\n```","ref":"usage.html#properties"},{"type":"extras","title":"Required Properties - Usage","doc":"By default, the properties defined by the properties keyword are not required.\nHowever, one can provide a list of `required` properties using the required\nkeyword.\n\n```elixir\niex> schema = Xema.new {:map, properties: %{foo: :string}, required: [:foo]}\niex> Xema.validate schema, %{foo: \"bar\"}\n:ok\niex> {:error, error} = Xema.validate schema, %{bar: \"foo\"}\n{:error, %Xema.ValidationError{\n  reason: %{required: [:foo]}\n}}\niex> Exception.message(error)\n\"Required properties are missing: [:foo].\"\n```","ref":"usage.html#required-properties"},{"type":"extras","title":"Additional Properties - Usage","doc":"The `additional_properties` keyword is used to control the handling of extra\nstuff, that is, properties whose names are not listed in the properties keyword.\nBy default any additional properties are allowed.\n\nThe `additional_properties` keyword may be either a boolean or an schema. If\n`additional_properties` is a boolean and set to false, no additional properties\nwill be allowed.\n\n```elixir\niex> schema = Xema.new {:map,\n...>   properties: %{foo: :string},\n...>   required: [:foo],\n...>   additional_properties: false\n...> }\niex> Xema.validate schema, %{foo: \"bar\"}\n:ok\niex> {:error, error} = Xema.validate schema, %{foo: \"bar\", bar: \"foo\"}\n{:error, %Xema.ValidationError{\n  reason: %{\n    properties: %{bar: %{additional_properties: false}}\n  }\n}}\niex> Exception.message(error)\n\"Expected only defined properties, got key [:bar].\"\n```\n\n`additional_properties` can also contain a schema to specify the type of\nadditional properites.\n\n```elixir\niex> schema = Xema.new {\n...>   :map,\n...>   properties: %{foo: :string},\n...>   additional_properties: :integer\n...> }\niex> Xema.valid? schema, %{foo: \"foo\", add: 1}\ntrue\niex> {:error, error} = Xema.validate schema, %{foo: \"foo\", add: \"one\"}\n{:error, %Xema.ValidationError{\n  reason: %{properties: %{add: %{type: :integer, value: \"one\"}}}\n}}\niex> Exception.message(error)\n~s|Expected :integer, got \"one\", at [:add].|\n```","ref":"usage.html#additional-properties"},{"type":"extras","title":"Pattern Properties - Usage","doc":"The keyword `pattern_properties` defined additional properties by regular\nexpressions.\n\n```elixir\niex> schema = Xema.new {\n...>   :map,\n...>   additional_properties: false,\n...>   pattern_properties: %{\n...>     ~r/^s_/ => :string,\n...>     ~r/^i_/ => :integer\n...>   }\n...> }\niex> Xema.valid? schema, %{\"s_0\" => \"foo\", \"i_1\" => 6}\ntrue\niex> Xema.valid? schema, %{s_0: \"foo\", i_1: 6}\ntrue\niex> {:error, error} = Xema.validate schema, %{s_0: \"foo\", f_1: 6.6}\n{:error, %Xema.ValidationError{\n  reason: %{properties: %{f_1: %{additional_properties: false}}}\n}}\niex> Exception.message(error)\n\"Expected only defined properties, got key [:f_1].\"\n```","ref":"usage.html#pattern-properties"},{"type":"extras","title":"Size - Usage","doc":"The number of properties on an object can be restricted using the\n`min_properties` and `max_properties` keywords.\n\n```elixir\niex> schema = Xema.new {:map,\n...>   min_properties: 2,\n...>   max_properties: 3\n...> }\niex> Xema.valid? schema, %{a: 1, b: 2}\ntrue\niex> {:error, error} = Xema.validate schema, %{}\n{:error, %Xema.ValidationError{\n  reason: %{min_properties: 2, value: %{}}\n}}\niex> Exception.message(error)\n\"Expected at least 2 properties, got %{}.\"\niex> {:error, error} = Xema.validate schema, %{a: 1, b: 2, c: 3, d: 4}\n{:error, %Xema.ValidationError{\n  reason: %{max_properties: 3, value: %{a: 1, b: 2, c: 3, d: 4}}\n}}\niex> Exception.message(error)\n\"Expected at most 3 properties, got %{a: 1, b: 2, c: 3, d: 4}.\"\n```","ref":"usage.html#size"},{"type":"extras","title":"Size - Usage","doc":"The type of a key in the schema also matters in validation.\n\n```elixir\niex> schema = Xema.new {:map,\n...>   properties: %{foo: :integer},\n...>   additional_properties: false\n...> }\niex> Xema.valid? schema, %{foo: 1}\ntrue\niex> {:error, error} = Xema.validate schema, %{\"foo\" => 1}\n{:error, %Xema.ValidationError{\n  reason: %{properties: %{\"foo\" => %{additional_properties: false}}}\n}}\niex> Exception.message(error)\n~s|Expected only defined properties, got key [\\\"foo\\\"].|\n```","ref":"usage.html#size"},{"type":"extras","title":"Dependencies - Usage","doc":"The `dependencies` keyword allows the schema of the object to change based on\nthe presence of certain special properties.\n\n```elixir\niex> schema = Xema.new {\n...>   :map,\n...>   properties: %{\n...>     a: :number,\n...>     b: :number,\n...>     c: :number\n...>   },\n...>   dependencies: %{\n...>     b: [:c]\n...>   }\n...> }\niex> Xema.valid? schema, %{a: 5}\ntrue\niex> Xema.valid? schema, %{c: 9}\ntrue\niex> Xema.valid? schema, %{b: 1}\nfalse\niex> Xema.valid? schema, %{b: 1, c: 7}\ntrue\n```","ref":"usage.html#dependencies"},{"type":"extras","title":"Type struct - Usage","doc":"Structs can also be validated.\n\n```elixir\niex> schema = Xema.new :struct\niex> Xema.valid? schema, ~r/.*/\ntrue\niex> Xema.valid? schema, %{}\nfalse\n```","ref":"usage.html#type-struct"},{"type":"extras","title":"Module - Usage","doc":"The `module` keyword allows specifying which struct is expected.\n\n```elixir\niex> schema = Xema.new {:struct, module: Regex}\niex> Xema.valid? schema, ~r/.*/\ntrue\niex> Xema.valid? schema, URI.parse(\"\")\nfalse\n```","ref":"usage.html#module"},{"type":"extras","title":"Map keywords for structs - Usage","doc":"The validations for `map` are also available.\n\nThe next example shows a schema for an URI that needs a fragment.\n\n```elixir\niex> schema = Xema.new {\n...>   :struct, module: URI, properties: %{fragment: :string}\n...> }\niex>\niex> Xema.valid?(schema, URI.parse(\"http://example.com\"))\nfalse\niex> Xema.valid?(schema, URI.parse(\"http://example.com#frag\"))\ntrue\n```","ref":"usage.html#map-keywords-for-structs"},{"type":"extras","title":"Multiples Types - Usage","doc":"`JSON Schema Draft: 4/6/7`\n\nIt is also possible to check if a value matches one of several types.\n\n```elixir\niex> schema = Xema.new {[:string, nil], min_length: 1}\niex> Xema.valid? schema, \"foo\"\ntrue\niex> Xema.valid? schema, nil\ntrue\niex> Xema.valid? schema, \"\"\nfalse\n```","ref":"usage.html#multiples-types"},{"type":"extras","title":"Allow Additional Types - Usage","doc":"`JSON Schema Draft: -`\n\nThe keyword `allow` adds an extra type to the schema validation.\n\n```elixir\niex> schema = Xema.new {:string, min_length: 1, allow: nil}\niex> Xema.valid? schema, \"foo\"\ntrue\niex> Xema.valid? schema, nil\ntrue\niex> Xema.valid? schema, \"\"\nfalse\n```","ref":"usage.html#allow-additional-types"},{"type":"extras","title":"Constants - Usage","doc":"`JSON Schema Draft: -/6/7`\n\nThis keyword checks if a value is equals to the given `const`.\n\n```elixir\niex> schema = Xema.new(const: 4711)\niex> Xema.validate schema, 4711\n:ok\niex> {:error, error} = Xema.validate schema, 333\n{:error, %Xema.ValidationError{\n  reason: %{const: 4711, value: 333}\n}}\niex> Exception.message(error)\n\"Expected 4711, got 333.\"\n```","ref":"usage.html#constants"},{"type":"extras","title":"Enumerations - Usage","doc":"The `enum` keyword is used to restrict a value to a fixed set of values. It must\nbe an array with at least one element, where each element is unique.\n\n```elixir\niex> schema = Xema.new {:any, enum: [1, \"foo\", :bar]}\niex> Xema.valid? schema, :bar\ntrue\niex> Xema.valid? schema, 42\nfalse\n```","ref":"usage.html#enumerations"},{"type":"extras","title":"Negate Schema - Usage","doc":"The keyword `not` negates a schema.\n\n```elixir\niex> schema = Xema.new(not: {:integer, minimum: 0})\niex> Xema.valid? schema, 10\nfalse\niex> Xema.valid? schema, -10\ntrue\n```","ref":"usage.html#negate-schema"},{"type":"extras","title":"If-Then-Else - Usage","doc":"The keywords `if`, `then`, `else` work together to implement conditional\napplication of a subschema based on the outcome of another subschema.\n\n```elixir\niex> schema =\n...>   Xema.new(\n...>     if: :list,\n...>     then: [items: :integer, min_items: 2],\n...>     else: :integer\n...>   )\n...>\n...> Xema.valid?(schema, 3)\ntrue\niex> Xema.valid?(schema, \"3\")\nfalse\niex> Xema.valid?(schema, [1])\nfalse\niex> Xema.valid?(schema, [1, 2])\ntrue\n```","ref":"usage.html#if-then-else"},{"type":"extras","title":"Custom validator - Usage","doc":"With the keyword `validator` a custom validator can be defined. The `validator`\nexpected a function or a tuple of module and function name. The validator\nfunction gets the current value and return :ok on success and an error tuple\non failure.\n\n```elixir\niex> defmodule Palindrome do\n...>   @xema Xema.new(\n...>     properties: %{\n...>       palindrome: {:string, validator: &Palindrome.check/1}\n...>     }\n...>   )\n...>\n...>   def check(value) do\n...>     case value == String.reverse(value) do\n...>       true -> :ok\n...>       false -> {:error, :no_palindrome}\n...>     end\n...>   end\n...>\n...>   def validate(value) do\n...>     Xema.validate(@xema, value)\n...>   end\n...> end\niex>\niex> Palindrome.validate(%{palindrome: \"abba\"})\n:ok\niex> {:error, error} = Palindrome.validate(%{palindrome: \"beatles\"})\n{:error, %Xema.ValidationError{\n  reason: %{\n    properties: %{\n      palindrome: %{validator: :no_palindrome, value: \"beatles\"}\n    }\n  }\n}}\niex> Exception.message(error)\n~s|Validator fails with :no_palindrome for value \"beatles\", at [:palindrome].|\n```","ref":"usage.html#custom-validator"},{"type":"extras","title":"Combine Schemas - Usage","doc":"The keywords `all_of`, `any_of`, and `one_of` combines schemas.\n\nWith `all_of` all schemas have to match.\n\n```elixir\niex> all = Xema.new(all_of: [\n...>   {:integer, multiple_of: 2},\n...>   {:integer, multiple_of: 3}\n...> ])\niex> 0..9 |> Enum.map(&Xema.valid?(all, &1)) |> Enum.with_index()\n[true: 0, false: 1, false: 2, false: 3, false: 4,\n false: 5, true: 6, false: 7, false: 8, false: 9]\n```\n\nWith `any_of` any schema have to match.\n\n```elixir\niex> any = Xema.new(any_of: [\n...>   {:integer, multiple_of: 2},\n...>   {:integer, multiple_of: 3}\n...> ])\niex> 0..9 |> Enum.map(&Xema.valid?(any, &1)) |> Enum.with_index()\n[true: 0, false: 1, true: 2, true: 3, true: 4,\n false: 5, true: 6, false: 7, true: 8, true: 9]\n```\n\nWith `one_of` exactly on schema have to match.\n\n```elixir\niex> one = Xema.new(one_of: [\n...>   {:integer, multiple_of: 2},\n...>   {:integer, multiple_of: 3}\n...> ])\niex> 0..9 |> Enum.map(&Xema.valid?(one, &1)) |> Enum.with_index()\n[false: 0, false: 1, true: 2, true: 3, true: 4,\n false: 5, false: 6, false: 7, true: 8, true: 9]\n```","ref":"usage.html#combine-schemas"},{"type":"extras","title":"Structuring a schema - Usage","doc":"This section will present some options to structuring and reuse schemas.\n\n### `definitions` and `ref`\n\nTo reuse a schema put it under the keyword `definitions`. Later on, the schema\ncan be referenced with the keyword `ref`.\n\n```elixir\niex> schema = Xema.new {\n...>   :map,\n...>   definitions: %{\n...>     positive: {:integer, minimum: 1},\n...>     negative: {:integer, maximum: -1}\n...>   },\n...>   properties: %{\n...>     a: {:ref, \"#/definitions/positive\"},\n...>     b: {:ref, \"#/definitions/positive\"},\n...>     c: {:ref, \"#/definitions/negative\"},\n...>     # You can also reference any other schema.\n...>     d: {:ref, \"#/properties/c\"}\n...>   }\n...> }\n...> Xema.validate schema, %{a: 1, c: -1}\n:ok\niex> {:error, error} = Xema.validate schema, %{b: 1, c: 1}\n{:error, %Xema.ValidationError{\n  reason: %{properties: %{c: %{maximum: -1, value: 1}}}\n}}\niex> Exception.message(error)\n\"Value 1 exceeds maximum value of -1, at [:c].\"\niex> Xema.validate schema, %{d: -1}\n:ok\niex> {:error, error} = Xema.validate schema, %{d: 1}\n{:error, %Xema.ValidationError{\n  reason: %{properties: %{d: %{maximum: -1, value: 1}}}\n}}\niex> Exception.message(error)\n\"Value 1 exceeds maximum value of -1, at [:d].\"\n```","ref":"usage.html#structuring-a-schema"},{"type":"extras","title":"Without `definitions` and `ref` - Usage","doc":"It is also possible to use a schema in another schema, as in the following code.\n\n```elixir\niex> positive = Xema.new {:integer, minimum: 1}\n...> negative = Xema.new {:integer, maximum: -1}\n...> schema = Xema.new {\n...>   :map,\n...>   properties: %{\n...>     a: positive,\n...>     b: positive,\n...>     c: negative\n...>   }\n...> }\n...> Xema.validate schema, %{a: 1, b: 2, c: -3}\n:ok\n```","ref":"usage.html#without-definitions-and-ref"},{"type":"extras","title":"In multiple files - Usage","doc":"To structure schemas in multiple files you have to configure a loader to load\nthe files. The section \"[Configure a loader](loader.html)\" described the\nconfiguration and implementation of an loader.\n\nLet's assume you have the following file available at\n`https://localhost:1234/positive.exon`.\n\n```elixir\n{\n  :string,\n  definitions: %{\n    or_nil: {:any_of, [:nil, {:ref, \"#\"}]}\n  }\n}\n```\n\nI admit this example is a little absurd.\nBut the URL could be also any other URL and with another loader, the file\ncould be on your hard disk.\n\nYou can use the schema above as follow.\n\n```elixir\niex> alias Test.RemoteLoaderExon\niex> schema = Xema.new({\n...>   :map,\n...>   id: \"http://localhost:1234\",\n...>   properties: %{\n...>     name: {:ref, \"xema_name.exon#/definitions/or_nil\"},\n...>     str: {:ref, \"xema_name.exon\"}\n...>   }\n...> }, loader: RemoteLoaderExon)\niex> Xema.validate schema, %{str: \"foo\"}\n:ok\niex> {:error, error} = Xema.validate schema, %{str: nil}\n{:error, %Xema.ValidationError{\n  reason: %{properties: %{str: %{type: :string, value: nil}}}\n}}\niex> Exception.message(error)\n\"Expected :string, got nil, at [:str].\"\niex> Xema.validate schema, %{name: nil}\n:ok\niex> Xema.validate schema, %{name: \"Funny van Dannen\"}\n:ok\niex> {:error, error} = Xema.validate schema, %{name: 66}\n{:error,\n  %Xema.ValidationError{\n    reason: %{\n      properties: %{\n        name: %{any_of: [\n          %{type: nil, value: 66},\n          %{type: :string, value: 66}\n        ],\n        value: 66}\n      }\n    }\n  }\n}\niex> Exception.message(error)\n\"\"\"\nNo match of any schema, at [:name].\n  Expected nil, got 66, at [:name].\n  Expected :string, got 66, at [:name].\\\n\"\"\"\n```","ref":"usage.html#in-multiple-files"},{"type":"extras","title":"Cast","doc":"# Cast\n\n`Xema` provides the functions `Xema.cast/2` and `Xema.cast!/2` to convert data according to the\nschema. The converted data will also be validated against the schema. The function `Xema.cast/2`\nreturns an error tuple with a `CastError` when the data cannot be converted.\n\n```elixir\niex> schema = Xema.new({:integer, minimum: 1})\niex> Xema.cast(schema, \"5\")\n{:ok, 5}\niex> Xema.cast(schema, \"-5\")\n{:error, %Xema.ValidationError{\n  reason: %{minimum: 1, value: -5}\n}}\niex> {:error, error} = Xema.cast(schema, [])\n{:error, %Xema.CastError{\n  key: nil,\n  path: [],\n  to: :integer,\n  value: []\n}}\niex> Exception.message(error)\n\"cannot cast [] to :integer\"\n```\n\nWith `use Xema` the functions are also available.\n\n```elixir\niex> defmodule Int do\n...>   use Xema\n...>\n...>   xema do: integer(minimum: 1)\n...> end\niex>\niex> Int.cast(\"6\")\n{:ok, 6}\niex> Int.cast(\"-1\")\n{:error, %Xema.ValidationError{\n  reason: %{minimum: 1, value: -1}\n}}\n```","ref":"cast.html"},{"type":"extras","title":"Maps - Cast","doc":"To convert map keys the schema has to specify the `keys` keyword.\n\n```elixir\niex> defmodule MapSchema do\n...>   use Xema\n...>\n...>   xema do\n...>     map(\n...>       keys: :atoms,\n...>       properties: %{\n...>         pos: integer(minimum: 0),\n...>         neg: integer(maximum: 0)\n...>       }\n...>     )\n...>   end\n...> end\niex>\niex> MapSchema.cast(%{\"pos\" => \"5\", \"neg\" => \"-5\"})\n{:ok, %{pos: 5, neg: -5}}\n```","ref":"cast.html#maps"},{"type":"extras","title":"Caster - Cast","doc":"To convert \"special\" structs a caster can be specified. A caster function has to return the\nconverted value in a tuple `{:ok, value}` or an `:error`.\n\n```elixir\niex> defmodule UrisSchema do\n...>   use Xema\n...>\n...>   def caster(string) when is_binary(string), do: {:ok, URI.parse(string)}\n...>\n...>   def caster(%URI{} = uri), do: {:ok, uri}\n...>\n...>   def caster(_), do: :error\n...>\n...>   xema do\n...>     map(\n...>       keys: :atoms,\n...>       properties: %{\n...>         uris: list(\n...>           items: strux(URI, caster: &UrisSchema.caster/1)\n...>         )\n...>       }\n...>     )\n...>   end\n...> end\niex> UrisSchema.cast(%{uris: [\"https://hexdocs.pm/elixir/Kernel.html\"]})\n{:ok, %{uris: [URI.parse(\"https://hexdocs.pm/elixir/Kernel.html\")]}}\n```\n\nThe caster can also be a tuple of module and function name or an mfa tuple.\n\nUsing `Caster` behaviour is also supported.\n\n```elixir\niex> defmodule UriCaster do\n...>   @behaviour Xema.Caster\n...>\n...>   @impl true\n...>   def cast(%URI{} = uri), do: {:ok, uri}\n...>\n...>   def cast(string) when is_binary(string), do: {:ok, URI.parse(string)}\n...>\n...>   def cast(_), do: :error\n...> end\niex>\niex> defmodule UriSchema do\n...>   use Xema\n...>\n...>   xema do\n...>     map(\n...>       keys: :strings,\n...>       properties: %{\n...>         \"uri\" => strux(URI, caster: UriCaster)\n...>       }\n...>     )\n...>   end\n...> end\niex>\niex> UriSchema.cast(%{uri: \"https://elixir-lang.org/docs.html\"})\n{:ok, %{\"uri\" => URI.parse(\"https://elixir-lang.org/docs.html\")}}\n```","ref":"cast.html#caster"},{"type":"extras","title":"Configure a loader","doc":"# Configure a loader\n\nA loader returns the data for a remote schema. The remote schemas are defined\nin a schema like this.\n\n```elixir\n...\n  properties: %{\n    int: {:ref, \"http://localhost:1234/int.exon\"}\n  }\n...\n```\n\nA loader will be configured like this.\n\n```elixir\nconfig :xema, loader: My.Loader\n```\n\nA loader is a module which use the behaviour `Xema.Loader`.\n\n```elixir\ndefmodule My.Loader do\n  @moduledoc false\n\n  @behaviour Xema.Loader\n\n  @spec fetch(binary) :: {:ok, map} | {:error, any}\n  def fetch(uri) do\n    with {:ok, response} <- get(uri), do: eval(response, uri)\n  end\n\n  defp get(uri) do\n    case HTTPoison.get(uri) do\n      {:ok, %HTTPoison.Response{status_code: 200, body: body}} ->\n        {:ok, body}\n\n      {:ok, %HTTPoison.Response{status_code: 404}} ->\n        {:error, \"Remote schema '#{uri}' not found.\"}\n\n      {:ok, %HTTPoison.Response{status_code: code}} ->\n        {:error, \"code: #{code}\"}\n\n      {:error, reason} ->\n        {:error, reason}\n    end\n  end\n\n  defp eval(str, uri) do\n    {data, _} = Code.eval_string(str)\n    {:ok, data}\n  rescue\n    error -> {:error, %{error | file: URI.to_string(uri)}}\n  end\nend\n```\n\nThe function `fetch/1` will be called by `Xema` and expects an `%URI{}`. The\nreturn value must be a tuple of `:ok` and the required data for a schema or an\nerror tuple.\n\n**Note!** This loader use `Code.eval_string/1` and eval is always evil.\n\n> **Warning:** string can be any Elixir code and will be executed with the same\n> privileges as the Erlang VM: this means that such code could compromise the\n> machine (for example by executing system commands). Don’t use `eval_string/3`\n> with untrusted input (such as strings coming from the network).\n> -- Elixir API","ref":"loader.html"},{"type":"extras","title":"File loader - Configure a loader","doc":"A loader to read schema from the local file system.\n\nIn the schema:\n```elixir\n...\n  properties: %{\n    int: {:ref, \"int.exon\"}\n  }\n...\n```\n\nThe loader:\n```elixir\ndefmodule My.Loader do\n  @moduledoc false\n\n  @behaviour Xema.Loader\n\n  @spec fetch(binary) :: {:ok, map} | {:error, any}\n  def fetch(uri) do\n    \"path/to/schemas\"\n    |> Path.join(uri.path)\n    |> File.read!()\n    |> eval(uri)\n  end\n\n  defp eval(str, uri) do\n    {data, _} = Code.eval_string(str)\n    {:ok, data}\n  rescue\n    error -> {:error, %{error | file: URI.to_string(uri)}}\n  end\nend\n```","ref":"loader.html#file-loader"},{"type":"extras","title":"JSON Schema loader - Configure a loader","doc":"If the scheme is created with `Xema.from_json_schema/2`, the loader must return\nthe decoded JSON.\n\nIn this case the file loader looks like this:\n```elixir\ndefmodule My.Loader do\n  @moduledoc false\n\n  @behaviour Xema.Loader\n\n  @spec fetch(binary) :: {:ok, map} | {:error, any}\n  def fetch(uri) do\n    \"path/to/schemas\"\n    |> Path.join(uri.path)\n    |> File.read!()\n    |> Jason.decode()\n  end\nend\n```","ref":"loader.html#json-schema-loader"},{"type":"extras","title":"Examples","doc":"# Examples\n\nA bunch of `Xema` examples.","ref":"examples.html"},{"type":"extras","title":"Basic - Examples","doc":"A minimal example.\n\n```elixir\niex> defmodule Example.Basic do\n...>   use Xema, multi: true, default: :foo\n...>\n...>   xema :person do\n...>     map(\n...>       properties: %{\n...>         first_name: :string,\n...>         last_name: :string,\n...>         age: {:integer, minimum: 0}\n...>       }\n...>     )\n...>   end\n...>\n...>   xema :foo, do: :string\n...> end\niex>\niex> Example.Basic.valid?(\n...>   :person,\n...>   %{first_name: \"James\", last_name: \"Brown\", age: 42}\n...> )\ntrue\n...> Example.Basic.valid?(\n...>   :person,\n...>   %{first_name: :james, last_name: \"Brown\", age: 42}\n...> )\nfalse\niex> {:error, error} = Example.Basic.validate(\n...>   :person,\n...>   %{first_name: :james, last_name: \"Brown\", age: 42}\n...> )\n{:error, %Xema.ValidationError{\n  reason: %{properties: %{first_name: %{type: :string, value: :james}}}\n}}\niex> Exception.message(error)\n\"Expected :string, got :james, at [:first_name].\"\niex> Example.Basic.valid?(\"foo\")\ntrue\niex> Example.Basic.valid?(:foo)\nfalse\n```","ref":"examples.html#basic"},{"type":"extras","title":"Options - Examples","doc":"An example to check opts.\n\n```elixir\niex> defmodule Example.Options do\n...>   use Xema\n...>\n...>   xema do\n...>     keyword(\n...>       properties: %{\n...>         foo: atom(enum: [:bar, :baz]),\n...>         limit: integer(minimum: 0),\n...>         msg: :string\n...>       },\n...>       required: [:foo, :limit],\n...>       additional_properties: false\n...>     )\n...>   end\n...> end\niex>\niex> Example.Options.validate(foo: :bar, limit: 11, msg: \"foo\")\n:ok\niex> {:error, error} = Example.Options.validate(foo: :foo, limit: 11)\n{:error, %Xema.ValidationError{\n  reason: %{properties: %{foo: %{enum: [:bar, :baz], value: :foo}}}\n}}\niex> Exception.message(error)\n\"Value :foo is not defined in enum, at [:foo].\"\niex> {:error, error} = Example.Options.validate(foo: :bar)\n{:error, %Xema.ValidationError{\n  reason: %{required: [:limit]}\n}}\niex> Exception.message(error)\n\"Required properties are missing: [:limit].\"\niex> {:error, error} = Example.Options.validate(foo: :bar, limit: 11, message: \"foo\")\n{:error, %Xema.ValidationError{\n  reason: %{properties: %{message: %{additional_properties: false}}}\n}}\niex> Exception.message(error)\n\"Expected only defined properties, got key [:message].\"\n```","ref":"examples.html#options"},{"type":"extras","title":"Custom validator - Examples","doc":"This example shows the use of an custom validator that is given as an tuple\nof module and function name.\n\n```elixir\niex> defmodule Example.Palindrome do\n...>   def check(str) do\n...>     case str == String.reverse(str) do\n...>       true -> :ok\n...>       false -> {:error, :no_palindrome}\n...>     end\n...>   end\n...> end\niex>\niex> defmodule Example.PaliSchema do\n...>   use Xema\n...>\n...>   xema :palindrome do\n...>     string(validator: {Example.Palindrome, :check})\n...>   end\n...> end\n...>\n...> Example.PaliSchema.valid?(:palindrome, \"racecar\")\ntrue\niex> Example.PaliSchema.valid?(:palindrome, \"bike\")\nfalse\niex> {:error, error} = Example.PaliSchema.validate(:palindrome, \"bike\")\n{:error, %Xema.ValidationError{\n  reason: %{validator: :no_palindrome, value: \"bike\"}\n}}\niex> Exception.message(error)\n~s|Validator fails with :no_palindrome for value \"bike\".|\n```\n\nA validator can also be specified as behaviour.\n\n```elixir\niex> defmodule Example.PalindromeB do\n...>   @behaviour Xema.Validator\n...>\n...>   @impl true\n...>   def validate(str) do\n...>     case str == String.reverse(str) do\n...>       true -> :ok\n...>       false -> {:error, :no_palindrome}\n...>     end\n...>   end\n...> end\niex>\niex> defmodule Example.PaliSchemaB do\n...>   use Xema\n...>\n...>   xema :palindrome do\n...>     string(validator: Example.PalindromeB)\n...>   end\n...> end\n...>\n...> Example.PaliSchemaB.valid?(:palindrome, \"racecar\")\ntrue\niex> Example.PaliSchemaB.valid?(:palindrome, \"bike\")\nfalse\niex> {:error, error} = Example.PaliSchemaB.validate(:palindrome, \"bike\")\n{:error, %Xema.ValidationError{\n  reason: %{validator: :no_palindrome, value: \"bike\"}\n}}\niex> Exception.message(error)\n~s|Validator fails with :no_palindrome for value \"bike\".|\n```\n\nThe custom validator can also be a part of the schema module.\n\n```elixir\niex> defmodule Example.Range do\n...>   use Xema\n...>\n...>   xema :range do\n...>     map(\n...>       properties: %{\n...>         from: integer(minimum: 0),\n...>         to: integer(maximum: 100)\n...>       },\n...>       validator: &Example.Range.check/1\n...>     )\n...>   end\niex>\niex>   def check(%{from: from, to: to}) do\n...>     case from         true -> :ok\n...>       false -> {:error, :from_greater_to}\n...>     end\n...>   end\n...> end\n...>\n...> Example.Range.validate(:range, %{from: 6, to: 8})\n:ok\niex> {:error, error} = Example.Range.validate(:range, %{from: 66, to: 8})\n{:error, %Xema.ValidationError{\n  reason: %{validator: :from_greater_to, value: %{from: 66, to: 8}}\n}}\niex> Exception.message(error)\n\"Validator fails with :from_greater_to for value %{from: 66, to: 8}.\"\niex> {:error, error} = Example.Range.validate(:range, %{from: 166, to: 118})\n{:error, %Xema.ValidationError{\n  reason: %{properties: %{to: %{maximum: 100, value: 118}}}\n}}\niex> Exception.message(error)\n\"Value 118 exceeds maximum value of 100, at [:to].\"\n```","ref":"examples.html#custom-validator"},{"type":"extras","title":"Cast JSON - Examples","doc":"The following example cast data structure that is decoded by `Jason.decode!/1`.\nFor the encoding of `URI` a `Jason.Encoder` implementation is needed.\n\n```elixir\ndefimpl Jason.Encoder, for: URI do\n  def encode(uri, _opts) do\n    ~s|\"#{URI.to_string(uri)}\"|\n  end\nend\n```\n\nThis example shows how a complex data structure decoded by a JSON parser can be\nconverted in a form described by a schema.\n\n```elixir\niex> defmodule CasterUri do\n...>   @behaviour Xema.Caster\n...>\n...>   @impl true\n...>   def cast(%URI{} = uri), do: {:ok, uri}\n...>\n...>   def cast(string) when is_binary(string), do: {:ok, URI.parse(string)}\n...>\n...>   def cast(_), do: :error\n...> end\niex>\niex> defmodule UserSchema do\n...>   use Xema\n...>\n...>   xema :user do\n...>     map(\n...>       keys: :atoms,\n...>       properties: %{\n...>         name: :string,\n...>         birthday: strux(Date),\n...>         favorites:\n...>           map(\n...>             keys: :atoms,\n...>             properties: %{\n...>               fruits: list(items: atom(enum: [:apple, :orange, :banana])),\n...>               uris: list(items: strux(URI, caster: CasterUri))\n...>             }\n...>           )\n...>       },\n...>       additional_properties: false\n...>     )\n...>   end\n...> end\niex>\niex> {:ok, json} =\n...>   %{\n...>     \"name\" => \"Nick\",\n...>     \"birthday\" => ~D|2000-04-17|,\n...>     \"favorites\" => %{\n...>       \"fruits\" => ~w(apple banana),\n...>       \"uris\" => [\"https://elixir-lang.org/\"]\n...>     }\n...>   }\n...>   |> UserSchema.cast!()\n...>   |> Jason.encode()\n{:ok, \"{\\\"birthday\\\":\\\"2000-04-17\\\",\\\"favorites\\\":{\\\"fruits\\\":[\\\"apple\\\",\\\"banana\\\"],\\\"uris\\\":[\\\"https://elixir-lang.org/\\\"]},\\\"name\\\":\\\"Nick\\\"}\"}\niex>\niex> json |> Jason.decode!() |> UserSchema.cast!()\n%{\n  birthday: ~D[2000-04-17],\n  favorites: %{\n    fruits: [:apple, :banana],\n    uris: [\n      %URI{\n        authority: \"elixir-lang.org\",\n        fragment: nil,\n        host: \"elixir-lang.org\",\n        path: \"/\",\n        port: 443,\n        query: nil,\n        scheme: \"https\",\n        userinfo: nil\n      }\n    ]\n  },\n  name: \"Nick\"\n}\n```","ref":"examples.html#cast-json"},{"type":"extras","title":"Struct - Examples","doc":"This example combines some schemas in a schema for a struct.\n\nThe first schema describes a key-value map with a string key and a value of type\nnumber or string.\n\n```elixir\ndefmodule ExApp.KeyValue do\n  use Xema\n\n  xema do\n    map(\n      keys: :strings,\n      additional_properties: [:number, :string],\n      property_names: [pattern: ~r/^[a-z][a-z_]*$/],\n      default: %{}\n    )\n  end\nend\n```\n\nThis schema is used as follows:\n\n```elixir\nassert KeyValue.valid?(%{\"str\" => \"Foo\", \"num\" => 5})\nassert KeyValue.cast(str: \"Foo\", num: 5) == {:ok, %{\"str\" => \"Foo\", \"num\" => 5}}\n```\n\nThe next schema is a simple struct schema.\n\n```elixir\ndefmodule ExApp.Location do\n  use Xema\n\n  xema_struct do\n    field :city, [:string, nil]\n    field :country, [:string, nil], min_length: 1\n  end\nend\n```\n\nWith a cast, a `Location` struct is returned.\n\n```elixir\nassert Location.cast(city: \"Berlin\") == {:ok, %Location{city: \"Berlin\", country: nil}}\n```\n\nThe `Grant` schema comes with two required fields.\n\n```elixir\ndefmodule ExApp.Grant do\n  use Xema\n\n  @ops [:foo, :bar, :baz]\n  @permissions [:create, :read, :update, :delete]\n\n  xema_struct do\n    field :op, :atom, enum: @ops\n    field :permissions, :list, items: {:atom, enum: @permissions}\n    required [:op, :permissions]\n  end\nend\n```\n\nThe example contains also a `Caster` for unix timestamps.\n\n```elixir\ndefmodule ExApp.UnixTimestamp do\n  @behaviour Xema.Caster\n\n  @impl true\n  def cast(timestamp) when is_integer(timestamp), do: {:ok, DateTime.from_unix!(timestamp)}\n\n  def cast(%DateTime{} = timestamp), do: timestamp\n\n  def cast(_), do: :error\nend\n```\n\nAll schemas above are use in the `User` schema.\n\n```elixir\ndefmodule ExApp.User do\n  use Xema\n\n  alias ExApp.{Grant, KeyValue, Location, UnixTimestamp}\n\n  @regex_uuid ~r/^[a-z0-9]{8}\\-[a-z0-9]{4}\\-[a-z0-9]{4}\\-[a-z0-9]{4}\\-[a-z0-9]{12}$/\n\n  xema_struct do\n    field :id, :string, default: {UUID, :uuid4}, pattern: @regex_uuid\n    field :name, :string, min_length: 1\n    field :age, [:integer, nil], minimum: 0\n    field :location, Location\n    field :grants, :list, items: Grant, default: []\n    field :settings, KeyValue\n    field :created, DateTime, caster: UnixTimestamp\n    field :updated, DateTime, caster: UnixTimestamp, allow: nil\n    required [:age]\n  end\nend\n```\n\nThis module used also `UUID` to set a default for the field `id`.\n\nA call of `User.cast!`\n\n```elixir\nExApp.User.cast!(\n  name: \"Nick\",\n  age: 21,\n  location: [city: \"Dortmud\", country: \"Germany\"],\n  grants: [%{op: :bar, permissions: [:read, :update]}],\n  settings: [foo: 44, bar: \"baz\"],\n  created: 1_567_922_779\n)\n```\n\nreturns a `User` struct\n\n```elixir\n%ExApp.User{\n  age: 21,\n  created: ~U[2019-09-08 06:06:19Z],\n  grants: [%ExApp.Grant{op: :bar, permissions: [:read, :update]}],\n  id: \"c5166552-25f5-43fe-91de-969344fd67d6\",\n  location: %ExApp.Location{city: \"Dortmud\", country: \"Germany\"},\n  name: \"Nick\",\n  settings: %{\"bar\" => \"baz\", \"foo\" => 44},\n  updated: nil\n}\n```","ref":"examples.html#struct"},{"type":"extras","title":"Validate with option `:fail` - Examples","doc":"With the option `:fail`, you can define when the validation is aborted. This\nalso influences how many error reasons are returned.\n- `:immediately` aborts the validation when the first validation fails.\n- `:early` (default) aborts on failed validations, but runs validations\n  for all properties and items.\n- `:finally` aborts after all possible validations.","ref":"examples.html#validate-with-option-fail"},{"type":"extras","title":"Examples - Examples","doc":"```elixir\niex> schema = Xema.new({:list, max_items: 3, items: :integer})\niex> data = [1, \"a\", \"b\"]\niex> {:error, error} = Xema.validate(schema, data, fail: :immediately)\niex> error.reason\n%{items: %{\n  1 => %{type: :integer, value: \"a\"}\n}}\niex> {:error, error} = Xema.validate(schema, data, fail: :early)\niex> error.reason\n%{items: %{\n  1 => %{type: :integer, value: \"a\"},\n  2 => %{type: :integer, value: \"b\"}\n}}\niex> {:error, error} = Xema.validate(schema, data, fail: :finally)\niex> error.reason\n%{items: %{\n  1 => %{type: :integer, value: \"a\"},\n  2 => %{type: :integer, value: \"b\"}\n}}\niex> # new data\niex> data = [1, \"a\", \"b\", 4]\niex> {:error, error} = Xema.validate(schema, data, fail: :immediately)\niex> error.reason\n%{max_items: 3, value: [1, \"a\", \"b\", 4]}\niex> {:error, error} = Xema.validate(schema, data, fail: :early)\niex> error.reason\n%{max_items: 3, value: [1, \"a\", \"b\", 4]}\niex> {:error, error} = Xema.validate(schema, data, fail: :finally)\niex> error.reason\n[\n  %{items: %{\n    1 => %{type: :integer, value: \"a\"},\n    2 => %{type: :integer, value: \"b\"}\n  }},\n  %{max_items: 3, value: [1, \"a\", \"b\", 4]}\n]\n```","ref":"examples.html#examples"},{"type":"extras","title":"Unsupported Features","doc":"# Unsupported Features\n\nThe keywords `readOnly` and `writeOnly` are not supported.\n\nThe keyword `contentMediaType` and `contentEncoding` are just for annotation and\nwill not be used for validation.\n\nThe following `format` checks are not supported:\n+ `idn-email` the I18N equivalent of `email`\n+ `idn-hostname` I18N equivalent of `hostname`\n+ `iri` the I18N equivalent of `uri`\n+ `iri-reference` the I18N equivalent of `uri-reference`","ref":"unsupported.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.17.5 - 2025/01/11 - Changelog","doc":"+ Fix warnings by conditionally compiling Decimal support.","ref":"changelog.html#0-17-5-2025-01-11"},{"type":"extras","title":"0.17.4 - 2024/07/04 - Changelog","doc":"+ Mark `Decimal` dep as optional again.","ref":"changelog.html#0-17-4-2024-07-04"},{"type":"extras","title":"0.17.3 - 2024/07/03 - Changelog","doc":"+ Fix typo in doc for `Xema.from_json_schema/2`.\n+ Fix spec for opts in `Xema.JsonSchema`.\n+ Remove all entries in `.dialyzer_ignore.exs`.\n+ Update test scripts to remove warnings when running with Elixir 1.17.","ref":"changelog.html#0-17-3-2024-07-03"},{"type":"extras","title":"0.17.2 - 2024/05/17 - Changelog","doc":"+ Update `Xema.ValidationError.format_error/1`.","ref":"changelog.html#0-17-2-2024-05-17"},{"type":"extras","title":"0.17.1 - 2023/07/28 - Changelog","doc":"+ Fix the ref inliner.","ref":"changelog.html#0-17-1-2023-07-28"},{"type":"extras","title":"0.17.0 - 2022/09/19 - Changelog","doc":"+ Require Elixir 1.8.0\n+ Update inspect implementations for `Xema.Ref` and `Xema.Schema`.","ref":"changelog.html#0-17-0-2022-09-19"},{"type":"extras","title":"0.16.1 - 2022/07/06 - Changelog","doc":"+ Add implementation to cast maps with integer keys to lists. Thanks to @egze.","ref":"changelog.html#0-16-1-2022-07-06"},{"type":"extras","title":"0.16.0 - 2022/05/11 - Changelog","doc":"+ Deprecated `@default true` to mark a schema as default.\n  To set a default add the option `:default` to `use Xema`.\n\n  Example:\n\n  ```elixir\n  defmoudle MyApp.Schemas do\n    use Xema, multi: true, default: :s1\n\n    xema :s1 do\n      ...\n    end\n\n    xema :s2 do\n      ...\n    end\n  end\n  ```\n+ Fix `$ref` applicator to accept none-keywords in the path.","ref":"changelog.html#0-16-0-2022-05-11"},{"type":"extras","title":"0.15.0 - 2022/01/09 - Changelog","doc":"+ Deprecate macro `Xema.Builder.xema/1` in combination with\n  `Xema.Builder.field/3`. Use `Xema.Builde.xema_struct/1` instead.\n\n  This change makes it clearer in which case a struct is created.\n  The functionality of `Xema.Builder.xema/1` and `Xema.Builder.xema/2` to create\n  simple schemas stays the same.","ref":"changelog.html#0-15-0-2022-01-09"},{"type":"extras","title":"Example: - Changelog","doc":"before 0.15.0:\n  ```elixir\n  defmoudle MyApp.User do\n    use Xema\n\n    xema do\n      field :name, :string\n    end\n  end\n  ```\n  since 0.15.0:\n  ```elixir\n  defmoudle MyApp.User do\n    use Xema\n\n    xema_struct do\n      field :name, :string\n    end\n  end\n  ```","ref":"changelog.html#example"},{"type":"extras","title":"0.14.3 - 2021/12/19 - Changelog","doc":"+ `Xeam.from_json_schema/2` sets now `keys: :strings` for any\n  `{\"type\": \"object\"}`","ref":"changelog.html#0-14-3-2021-12-19"},{"type":"extras","title":"0.14.2 - 2021/10/09 - Changelog","doc":"+ Fix casting to struct.","ref":"changelog.html#0-14-2-2021-10-09"},{"type":"extras","title":"0.14.1 - 2021/10/01 - Changelog","doc":"+ Fix casting to `struct` with required properties.","ref":"changelog.html#0-14-1-2021-10-01"},{"type":"extras","title":"0.14.0 - 2021/08/07 - Changelog","doc":"","ref":"changelog.html#0-14-0-2021-08-07"},{"type":"extras","title":"Breaking changes - Changelog","doc":"+ Refactoring the cast functionality with the combining operators `:any_of` and\n  `:one_of`. Before 0.14.0 `Xema` tries to cast as many values from a data\n  structure according to all sub-schemas as possible.\n\n  Before 0.14.0 `Xema` tries to cast as many values from a data structure\n  according to all sub-schemas as possible. For this, any casting result in\n  `:any_of` and `one_of` was the input for the next casting. The new version\n  takes the first valid cast as a result of the whole combining operator.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.13.10 - 2021/06/05 - Changelog","doc":"+ Fix: Add types with option `:allow`.","ref":"changelog.html#0-13-10-2021-06-05"},{"type":"extras","title":"0.13.9 - 2021/06/01 - Changelog","doc":"+ Fix: Cast to boolean not working as expected.","ref":"changelog.html#0-13-9-2021-06-01"},{"type":"extras","title":"0.13.8 - 2021/05/01 - Changelog","doc":"+ Use the `:default` option to set `defstruct` default. Thanks to @garret-smith.","ref":"changelog.html#0-13-8-2021-05-01"},{"type":"extras","title":"0.13.7 - 2021/03/14 - Changelog","doc":"+ Fix inlining","ref":"changelog.html#0-13-7-2021-03-14"},{"type":"extras","title":"0.13.6 - 2020/12/23 - Changelog","doc":"+ Add `.formatter.ex` to package","ref":"changelog.html#0-13-6-2020-12-23"},{"type":"extras","title":"0.13.5 - 2020/10/26 - Changelog","doc":"+ Adding `Xema.Builder.all_of/2`, `Xema.Builder.any_of/2` and\n  `Xema.Builder.one_of/2`","ref":"changelog.html#0-13-5-2020-10-26"},{"type":"extras","title":"0.13.4 - 2020/10/14 - Changelog","doc":"+ Updating `decimal` dependency.","ref":"changelog.html#0-13-4-2020-10-14"},{"type":"extras","title":"0.13.3 - 2020/10/12 - Changelog","doc":"+ Relaxing format validation for URIs.\n+ Fixing keyword `:items`.","ref":"changelog.html#0-13-3-2020-10-12"},{"type":"extras","title":"0.13.2 - 2020/10/06 - Changelog","doc":"+ Fixing docs for `Xema.Builder.strux/1`.\n+ Updating deps.\n+ Fixing tests.","ref":"changelog.html#0-13-2-2020-10-06"},{"type":"extras","title":"0.13.1 - 2020/09/09 - Changelog","doc":"+ Fixing minor issues.","ref":"changelog.html#0-13-1-2020-09-09"},{"type":"extras","title":"0.13.0 - 2020/09/07 - Changelog","doc":"+ Adding option `:fail` to `Xema.validate/3`. See also\n  [\"Validate with option `:fail`\"](examples.html#validate-with-option-fail)","ref":"changelog.html#0-13-0-2020-09-07"},{"type":"extras","title":"0.12.0 - 2020/08/26 - Changelog","doc":"+ Refactoring of the properties validation.","ref":"changelog.html#0-12-0-2020-08-26"},{"type":"extras","title":"Breaking changes - Changelog","doc":"+ The error data structure for `list` `items` has changed from a `list` of\n  `tuple`s to a `map`. This change allows you to convert `reason` directly to\n  JSON.\n\nsince 0.12.0:\n```elixir\niex> schema = Xema.new({:list, items: :integer})\niex> Xema.validate(schema, [1, \"foo\", 2, :bar])\n{:error,\n %Xema.ValidationError{\n   message: nil,\n   reason: %{\n     items: %{\n       1 => %{type: :integer, value: \"foo\"},\n       3 => %{type: :integer, value: :bar}\n     }\n   }\n }}\n```\nbefore 0.12.0:\n```elixir\niex> schema = Xema.new({:list, items: :integer})\niex> Xema.validate(schema, [1, \"foo\", 2, :bar])\n{:error,\n %Xema.ValidationError{\n   message: nil,\n   reason: %{\n     items: [\n       {1, %{type: :integer, value: \"foo\"}},\n       {3, %{type: :integer, value: :bar}}\n     ]\n   }\n }}\n```","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.11.4 - 2020/08/19 - Changelog","doc":"+ Fix an issue with circular references.","ref":"changelog.html#0-11-4-2020-08-19"},{"type":"extras","title":"0.11.3 - 2020/08/18 - Changelog","doc":"+ Fix an issue with circular references.","ref":"changelog.html#0-11-3-2020-08-18"},{"type":"extras","title":"0.11.2 - 2020/07/31 - Changelog","doc":"+ Fix a bug when `unique_items` is explicit set to `false`.","ref":"changelog.html#0-11-2-2020-07-31"},{"type":"extras","title":"0.11.1 - 2020/07/17 - Changelog","doc":"+ Fix hostname format validation. Thanks to @romul.","ref":"changelog.html#0-11-1-2020-07-17"},{"type":"extras","title":"0.11.0 - 2019/10/19 - Changelog","doc":"+ Add `Xema.from_json_schema`.","ref":"changelog.html#0-11-0-2019-10-19"},{"type":"extras","title":"0.10.0 - 2019/09/8 - Changelog","doc":"+ Refactor `Xema.cast` with an improvement of the error messages.\n+ Add macro `field` and `required` for struct schemas.","ref":"changelog.html#0-10-0-2019-09-8"},{"type":"extras","title":"0.9.3 - 2019/08/25 - Changelog","doc":"+ Fix specs.","ref":"changelog.html#0-9-3-2019-08-25"},{"type":"extras","title":"0.9.2 - 2019/08/21 - Changelog","doc":"+ Update specs.","ref":"changelog.html#0-9-2-2019-08-21"},{"type":"extras","title":"0.9.1 - 2019/06/16 - Changelog","doc":"+ Silence warnings for `Decimal`.","ref":"changelog.html#0-9-1-2019-06-16"},{"type":"extras","title":"0.9.0 - 2019/06/16 - Changelog","doc":"","ref":"changelog.html#0-9-0-2019-06-16"},{"type":"extras","title":"New features - Changelog","doc":"+ Add `Xema.cast/2`.","ref":"changelog.html#new-features"},{"type":"extras","title":"Breaking changes - Changelog","doc":"+ Add option `multi` to use multiple schema modules: `use Xema, multi: true`.\n+ Remove `xema/2` macro.\n+ Add `xema/0` and `xema/1` macro.\n+ Change return value of `Xema.validate/2` to `{:error, Exception.t}` in case\n  of error.\n+ Update `Xema.ValidationError`.\n  + `ValidationError.reason` contains the error map of the previous versions\n    with some changes.\n  + `ValidationError.message` contains an error message.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bugfixes - Changelog","doc":"+ Fixes in `Xema.Validator`. In some cases the validators `any_of`, `one_of`,\n`all_of`, `enum`, `not`, `const`, and `if-then-else` was be ignored.","ref":"changelog.html#bugfixes"},{"type":"extras","title":"O.8.1 - 2019/06/02 - Changelog","doc":"+ Fix email format checker.","ref":"changelog.html#o-8-1-2019-06-02"},{"type":"extras","title":"0.8.0 - 2019/03/10 - Changelog","doc":"+ Inline references in schemas by default. Increased performance of reference\n  handling by 30%.\n+ Rename `defxema` to `xema` and refactor macros.\n+ Add custom `validator`.\n+ Add examples page.\n+ Add `@default true`.","ref":"changelog.html#0-8-0-2019-03-10"},{"type":"extras","title":"0.7.0 - 2019/02/10 - Changelog","doc":"+ Increase `ref` performance.\n+ Fix option `:resolver` in `Xema.new/2`.\n+ Refactor `ref` handling,\n+ Rename `resolver` to `loader`.\n+ Remove deprecated function `Xema.is_valid/2`.\n+ Reduce public API docs.\n+ Add `Xema.Builder` and `use Xema`.","ref":"changelog.html#0-7-0-2019-02-10"},{"type":"extras","title":"0.6.3 - 2019/01/15 - Changelog","doc":"+ Ignore unknown formatters.","ref":"changelog.html#0-6-3-2019-01-15"},{"type":"extras","title":"0.6.2 - 2019/01/13 - Changelog","doc":"+ Change regex for email validation (~7x faster).\n+ Add `:resolver` option to `Xema.new`.\n+ Speed up string validator.","ref":"changelog.html#0-6-2-2019-01-13"},{"type":"extras","title":"0.6.1 - 2019/01/06 - Changelog","doc":"Update docs.","ref":"changelog.html#0-6-1-2019-01-06"},{"type":"extras","title":"0.6.0 - 2018/12/29 - Changelog","doc":"+ Fixed and updated some specs.\n+ Remote check for references is moved to the `behaviour`.\n+ Key types in the schema are now matters for validation. See\n  [Usage - Key types](https://hexdocs.pm/xema/usage.html#key_types)","ref":"changelog.html#0-6-0-2018-12-29"},{"type":"extras","title":"0.5.0 - 2018/12/25 - Changelog","doc":"+ The function `Xema.is_valid?/2` is deprecated. Use `Xema.valid?/2` instead.\n+ Add keyword `const`.\n+ Add keywords `if`, `then`, `else`.\n+ Add handling for none-keyword data.\n+ Add annotation keywords\n  + `examples`\n  + `comment`\n  + `contentEncoding`\n  + `contentMediaType`contentMediaType\n+ Add new `format` checks.\n+ Add validatiors for `atom`, `keyword`, `tuple` and `struct`\n+ Add schema validator to validate data give to `Xema.new/1`.\n+ `Xema.new/2` becomes `Xema.new/1`.\n  Migrate to 0.5.0:\n  ```elixir\n  # < 0.5.0\n  Xema.new(:integer, minimum: 0)\n  # >= 0.5.0\n  Xema.new({:integer, minimum: 0})\n  ```\n+ Add Xema.validate!/2","ref":"changelog.html#0-5-0-2018-12-25"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}